#ifndef TD3DDP_PtrStrideH
#define TD3DDP_PtrStrideH

// ==========================================================================
// File:         TD3DDP_PtrStride.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DDP_PtrStride Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DDP_PtrStride
// Description: The TD3DDP_PtrStride component wraps multiple DirectX 
//              D3DDP_PTRSTRIDE structures, each of which references an array 
//              of vertex components and a stride value to the next array 
//              element. 
//              
//              This component is used by the TD3DDrawPrimitiveStridedData 
//              component. 
//              
//              When used with a composite vertex format like D3DLVERTEX, the 
//              Data property references a particular component and the 
//              Stride property represents the stride, in bytes, of the 
//              composite format. 
//              When Data references a single vertex, the Stride property 
//              represents the stride, in bytes, of each element in the 
//              array. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DDP_PtrStride : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DDP_PtrStride : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DDP_PtrStride method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DDP_PtrStride component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DDP_PTRSTRIDE structures wrapped inside the 
    //              TD3DDP_PtrStride component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DDP_PtrStride component wraps a single 
    //              D3DDP_PTRSTRIDE structure. To wrap multiple structures, 
    //              set the TD3DDP_PtrStride::ArraySize to the total number 
    //              of internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Data
    // Description: The Data property references a composite vertex format or 
    //              an array of distinct vertex components. 
    // ----------------------------------------------------------------------
    __property void* Data[dword pArrayIndex] = { read=FGetData, 
    write=FSetData }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DDP_PTRSTRIDE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DDP_PTRSTRIDE structures. This result is the 
    //              same as multiplying TD3DDP_PtrStride::ArraySize and 
    //              TD3DDP_PtrStride::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Stride
    // Description: The Stride property defines the stride, in bytes, of the 
    //              composite vertex format, or the stride, in bytes, of each 
    //              element in the array of distinct vertex components. 
    // ----------------------------------------------------------------------
    __property dword Stride[dword pArrayIndex] = { read=FGetStride, 
    write=FSetStride }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DDP_PtrStride's internal D3DDP_PTRSTRIDE 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DDP_PtrStride::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DDP_PTRSTRIDE structures of the 
    //              TD3DDP_PtrStride component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DDP_PtrStride(TComponent* Owner); 
    virtual __fastcall ~TD3DDP_PtrStride(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DDP_PTRSTRIDE* Internal_D3DDP_PTRSTRIDE_Ptr = { 
    read=FGetInternal_D3DDP_PTRSTRIDE_Ptr, nodefault }; 

    void __fastcall Internal_D3DDP_PTRSTRIDE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    void* __fastcall FGetData( dword pArrayIndex ); 
    void __fastcall FSetData( dword pArrayIndex, void* pData ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    dword __fastcall FGetStride( dword pArrayIndex ); 
    void __fastcall FSetStride( dword pArrayIndex, dword pStride ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DDP_PTRSTRIDE* __fastcall FGetInternal_D3DDP_PTRSTRIDE_Ptr(); 
    void __fastcall FSetInternal_D3DDP_PTRSTRIDE( dword pArrayIndex, 
    D3DDP_PTRSTRIDE* pD3DDP_PTRSTRIDE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DDP_PTRSTRIDE* fD3DDP_PTRSTRIDE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
