#ifndef TD3DDevInfo_TextureManagerH
#define TD3DDevInfo_TextureManagerH

// ==========================================================================
// File:         TD3DDevInfo_TextureManager.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DDevInfo_TextureManager Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DDevInfo_TextureManager
// Description: The TD3DDevInfo_TextureManager component wraps the DirectX 
//              D3DDEVINFO_TEXTUREMANAGER structure which describes the 
//              status of the texture manager. 
//              
//              This component is used in the TDx_3DDevice::GetInfo() method 
//              to report the current state of the texture manager.   
//              Statistics on the numbers of textures and the amount of 
//              memory being used by them are available. 
//              
//              In particular, NumUsedTexInVid and NumTexturesUsed can be 
//              used to calculate the percentage of textures in the last 
//              frame that were resident in video memory. 
//              ie: Percentage = (NumUsedTexInVid * 100) / NumTexturesUsed 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DDevInfo_TextureManager : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DDevInfo_TextureManager : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    LastPri
    // Description: The LastPri property contains the priority of the last 
    //              texture removed from memory. 
    // ----------------------------------------------------------------------
    __property dword LastPri = { read=FGetLastPri, write=FSetLastPri, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumEvicts
    // Description: The NumEvicts property contains the number of textures 
    //              removed during the last frame. 
    // ----------------------------------------------------------------------
    __property dword NumEvicts = { read=FGetNumEvicts, write=FSetNumEvicts, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumTexturesUsed
    // Description: The NumTexturesUsed contains the total number of textures 
    //              used in the last frame. 
    // ----------------------------------------------------------------------
    __property dword NumTexturesUsed = { read=FGetNumTexturesUsed, 
    write=FSetNumTexturesUsed, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumUsedTexInVid
    // Description: The NumUsedTexInVid property contains how many video 
    //              memory textures were used in the last frame. 
    // ----------------------------------------------------------------------
    __property dword NumUsedTexInVid = { read=FGetNumUsedTexInVid, 
    write=FSetNumUsedTexInVid, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumVidCreates
    // Description: The NumVidCreates property contains how many video memory 
    //              textures were created during the last frame. 
    // ----------------------------------------------------------------------
    __property dword NumVidCreates = { read=FGetNumVidCreates, 
    write=FSetNumVidCreates, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Thrashing
    // Description: The Thrashing property indicates whether thrashing 
    //              occured in the last frame. 
    // ----------------------------------------------------------------------
    __property bool Thrashing = { read=FGetThrashing, write=FSetThrashing, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    TotalBytes
    // Description: The TotalBytes property defines how much memory, in 
    //              bytes, that is allocated to managed textures. 
    // ----------------------------------------------------------------------
    __property dword TotalBytes = { read=FGetTotalBytes, 
    write=FSetTotalBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    TotalManaged
    // Description: The TotalManaged property defines how many textures are 
    //              currently being managed. 
    // ----------------------------------------------------------------------
    __property dword TotalManaged = { read=FGetTotalManaged, 
    write=FSetTotalManaged, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    WorkingSet
    // Description: The WorkingSet property defines how many textures are 
    //              currently in video memory. 
    // ----------------------------------------------------------------------
    __property dword WorkingSet = { read=FGetWorkingSet, 
    write=FSetWorkingSet, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    WorkingSetBytes
    // Description: The WorkingSetBytes property defines how much video 
    //              memory, in bytes, is being used by resident textures. 
    // ----------------------------------------------------------------------
    __property dword WorkingSetBytes = { read=FGetWorkingSetBytes, 
    write=FSetWorkingSetBytes, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DDevInfo_TextureManager component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DDevInfo_TextureManager 
    //              method or fget/fset. eg. D3D_OK or DDERR_SURFACELOST or 
    //              TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DDEVINFO_TEXTUREMANAGER structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DDevInfo_TextureManager's internal 
    //              D3DDEVINFO_TEXTUREMANAGER structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DDevInfo_TextureManager(TComponent* Owner); 
    virtual __fastcall ~TD3DDevInfo_TextureManager(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DDEVINFO_TEXTUREMANAGER* 
    Internal_D3DDEVINFO_TEXTUREMANAGER_Ptr = { 
    read=FGetInternal_D3DDEVINFO_TEXTUREMANAGER_Ptr, nodefault }; 

    void __fastcall Internal_D3DDEVINFO_TEXTUREMANAGER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetLastPri(); 
    void __fastcall FSetLastPri( dword pLastPri ); 
    dword __fastcall FGetNumEvicts(); 
    void __fastcall FSetNumEvicts( dword pNumEvicts ); 
    dword __fastcall FGetNumTexturesUsed(); 
    void __fastcall FSetNumTexturesUsed( dword pNumTexturesUsed ); 
    dword __fastcall FGetNumUsedTexInVid(); 
    void __fastcall FSetNumUsedTexInVid( dword pNumUsedTexInVid ); 
    dword __fastcall FGetNumVidCreates(); 
    void __fastcall FSetNumVidCreates( dword pNumVidCreates ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    bool __fastcall FGetThrashing(); 
    void __fastcall FSetThrashing( bool pThrashing ); 
    dword __fastcall FGetTotalBytes(); 
    void __fastcall FSetTotalBytes( dword pTotalBytes ); 
    dword __fastcall FGetTotalManaged(); 
    void __fastcall FSetTotalManaged( dword pTotalManaged ); 
    dword __fastcall FGetWorkingSet(); 
    void __fastcall FSetWorkingSet( dword pWorkingSet ); 
    dword __fastcall FGetWorkingSetBytes(); 
    void __fastcall FSetWorkingSetBytes( dword pWorkingSetBytes ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DDEVINFO_TEXTUREMANAGER* __fastcall 
    FGetInternal_D3DDEVINFO_TEXTUREMANAGER_Ptr(); 
    void __fastcall FSetInternal_D3DDEVINFO_TEXTUREMANAGER( 
    D3DDEVINFO_TEXTUREMANAGER* pD3DDEVINFO_TEXTUREMANAGER ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DDEVINFO_TEXTUREMANAGER fD3DDEVINFO_TEXTUREMANAGER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
