#ifndef TD3DLVertexH
#define TD3DLVertexH

// ==========================================================================
// File:         TD3DLVertex.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DLVertex Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DLVertex
// Description: The TD3DLVertex component wraps multiple DirectX D3DLVERTEX 
//              structures defining untransformed and lit vertices in terms 
//              of their model coordinates and color. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DLVertex : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DLVertex : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DLVertex component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DLVERTEX structures wrapped inside the TD3DLVertex 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DLVertex component wraps a single D3DLVERTEX 
    //              structure. To wrap multiple structures, set the 
    //              TD3DLVertex::ArraySize to the total number of internal 
    //              structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Color
    // Description: The Color property contains a D3DCOLOR defining the color 
    //              component of the vertex. 
    //              
    //              This property is unioned with the color property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR Color[dword pArrayIndex] = { read=FGetColor, 
    write=FSetColor }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DLVertex method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DLVERTEX structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DLVERTEX structures. This result is the same 
    //              as multiplying TD3DLVertex::ArraySize and 
    //              TD3DLVertex::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Specular
    // Description: The Specular property contains a D3DCOLOR defining the 
    //              specular component of the vertex. 
    //              
    //              This property is unioned with the specular property of 
    //              this object. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR Specular[dword pArrayIndex] = { read=FGetSpecular, 
    write=FSetSpecular }; 

    // ----------------------------------------------------------------------
    // Property:    TU
    // Description: The TU property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the tu property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TU[dword pArrayIndex] = { read=FGetTU, write=FSetTU 
    }; 

    // ----------------------------------------------------------------------
    // Property:    TV
    // Description: The TV property contains a D3DVALUE specifying the V 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the tv property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TV[dword pArrayIndex] = { read=FGetTV, write=FSetTV 
    }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property contains a D3DVALUE specifying the X model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the x property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE X[dword pArrayIndex] = { read=FGetX, write=FSetX }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property contains a D3DVALUE specifying the X model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the y property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Y[dword pArrayIndex] = { read=FGetY, write=FSetY }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property contains a D3DVALUE specifying the X model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the z property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Z[dword pArrayIndex] = { read=FGetZ, write=FSetZ }; 

    // ----------------------------------------------------------------------
    // Property:    color
    // Description: The color property contains a D3DCOLOR defining the color 
    //              component of the vertex. 
    //              
    //              This property is unioned with the Color property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR color[dword pArrayIndex] = { read=FGetcolor, 
    write=FSetcolor }; 

    // ----------------------------------------------------------------------
    // Property:    specular
    // Description: The specular property contains a D3DCOLOR defining the 
    //              specular component of the vertex. 
    //              
    //              This property is unioned with the Specular property of 
    //              this object. 
    // ----------------------------------------------------------------------
    __property D3DCOLOR specular[dword pArrayIndex] = { read=FGetspecular, 
    write=FSetspecular }; 

    // ----------------------------------------------------------------------
    // Property:    tu
    // Description: The tu property contains a D3DVALUE specifying the U 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the TU property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tu[dword pArrayIndex] = { read=FGettu, write=FSettu 
    }; 

    // ----------------------------------------------------------------------
    // Property:    tv
    // Description: The tv property contains a D3DVALUE specifying the V 
    //              texture coordinate of the vertex. 
    //              
    //              This property is unioned with the TV property of this 
    //              object. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tv[dword pArrayIndex] = { read=FGettv, write=FSettv 
    }; 

    // ----------------------------------------------------------------------
    // Property:    x
    // Description: The x property contains a D3DVALUE specifying the X model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the X property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE x[dword pArrayIndex] = { read=FGetx, write=FSetx }; 

    // ----------------------------------------------------------------------
    // Property:    y
    // Description: The y property contains a D3DVALUE specifying the Y model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the Y property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE y[dword pArrayIndex] = { read=FGety, write=FSety }; 

    // ----------------------------------------------------------------------
    // Property:    z
    // Description: The z property contains a D3DVALUE specifying the Z model 
    //              coordinate of the vertex. 
    //              
    //              This property is unioned with the Z property of this 
    //              object. 
    //              
    //              The vertex position is defined relative to the origin 
    //              specified for the objects model space. 
    // ----------------------------------------------------------------------
    __property D3DVALUE z[dword pArrayIndex] = { read=FGetz, write=FSetz }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DLVertex's internal D3DLVERTEX structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DLVertex::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DLVERTEX structures of the 
    //              TD3DLVertex component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DLVertex(TComponent* Owner); 
    virtual __fastcall ~TD3DLVertex(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DLVERTEX* Internal_D3DLVERTEX_Ptr = { 
    read=FGetInternal_D3DLVERTEX_Ptr, nodefault }; 

    void __fastcall Internal_D3DLVERTEX_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    D3DCOLOR __fastcall FGetColor( dword pArrayIndex ); 
    void __fastcall FSetColor( dword pArrayIndex, D3DCOLOR pColor ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    D3DCOLOR __fastcall FGetSpecular( dword pArrayIndex ); 
    void __fastcall FSetSpecular( dword pArrayIndex, D3DCOLOR pSpecular ); 
    D3DVALUE __fastcall FGetTU( dword pArrayIndex ); 
    void __fastcall FSetTU( dword pArrayIndex, D3DVALUE pTU ); 
    D3DVALUE __fastcall FGetTV( dword pArrayIndex ); 
    void __fastcall FSetTV( dword pArrayIndex, D3DVALUE pTV ); 
    D3DVALUE __fastcall FGetX( dword pArrayIndex ); 
    void __fastcall FSetX( dword pArrayIndex, D3DVALUE pX ); 
    D3DVALUE __fastcall FGetY( dword pArrayIndex ); 
    void __fastcall FSetY( dword pArrayIndex, D3DVALUE pY ); 
    D3DVALUE __fastcall FGetZ( dword pArrayIndex ); 
    void __fastcall FSetZ( dword pArrayIndex, D3DVALUE pZ ); 
    D3DCOLOR __fastcall FGetcolor( dword pArrayIndex ); 
    void __fastcall FSetcolor( dword pArrayIndex, D3DCOLOR pcolor ); 
    D3DCOLOR __fastcall FGetspecular( dword pArrayIndex ); 
    void __fastcall FSetspecular( dword pArrayIndex, D3DCOLOR pspecular ); 
    D3DVALUE __fastcall FGettu( dword pArrayIndex ); 
    void __fastcall FSettu( dword pArrayIndex, D3DVALUE ptu ); 
    D3DVALUE __fastcall FGettv( dword pArrayIndex ); 
    void __fastcall FSettv( dword pArrayIndex, D3DVALUE ptv ); 
    D3DVALUE __fastcall FGetx( dword pArrayIndex ); 
    void __fastcall FSetx( dword pArrayIndex, D3DVALUE px ); 
    D3DVALUE __fastcall FGety( dword pArrayIndex ); 
    void __fastcall FSety( dword pArrayIndex, D3DVALUE py ); 
    D3DVALUE __fastcall FGetz( dword pArrayIndex ); 
    void __fastcall FSetz( dword pArrayIndex, D3DVALUE pz ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DLVERTEX* __fastcall FGetInternal_D3DLVERTEX_Ptr(); 
    void __fastcall FSetInternal_D3DLVERTEX( dword pArrayIndex, D3DLVERTEX* 
    pD3DLVERTEX ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DLVERTEX* fD3DLVERTEX; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
