#ifndef TD3DMatrixH
#define TD3DMatrixH

// ==========================================================================
// File:         TD3DMatrix.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DMatrix Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DMatrix
// Description: The TD3DMatrix component wraps the DirectX D3DMATRIX 
//              structure, which describes a matrix used for transformations. 
//              
//              This component is used by the TDx_3DDevice::GetTransform(), 
//              TDx_3DDevice::MultiplyTransform() and 
//              TDx_3DDevice::SetTransform() methods in the various world, 
//              view, projection and texture stage transformations. 
//              The matrices are 4 by 4 homogenous matrices, listed as (row, 
//              column). eg: The property _32 represents row 3, column 2 or 
//              (3,2). 
//              
//              In a projection matrix, the _34 property cannot be a negative 
//              number. If a negative value is needed in this location, scale 
//              the entire matrix by -1 instead. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DMatrix : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DMatrix : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DMatrix component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DMatrix method or fget/fset. 
    //              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal D3DMATRIX structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _11
    // Description: The _11 property contains the (1,1) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _11 = { read=FGet_11, write=FSet_11, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _12
    // Description: The _12 property contains the (1,2) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _12 = { read=FGet_12, write=FSet_12, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _13
    // Description: The _13 property contains the (1,3) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _13 = { read=FGet_13, write=FSet_13, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _14
    // Description: The _14 property contains the (1,4) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _14 = { read=FGet_14, write=FSet_14, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _21
    // Description: The _21 property contains the (2,1) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _21 = { read=FGet_21, write=FSet_21, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _22
    // Description: The _22 property contains the (2,2) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _22 = { read=FGet_22, write=FSet_22, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _23
    // Description: The _23 property contains the (2,3) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _23 = { read=FGet_23, write=FSet_23, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _24
    // Description: The _24 property contains the (2,4) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _24 = { read=FGet_24, write=FSet_24, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _31
    // Description: The _31 property contains the (3,1) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _31 = { read=FGet_31, write=FSet_31, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _32
    // Description: The _32 property contains the (3,2) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _32 = { read=FGet_32, write=FSet_32, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _33
    // Description: The _33 property contains the (3,3) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _33 = { read=FGet_33, write=FSet_33, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _34
    // Description: The _34 property contains the (3,4) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _34 = { read=FGet_34, write=FSet_34, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _41
    // Description: The _41 property contains the (4,1) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _41 = { read=FGet_41, write=FSet_41, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _42
    // Description: The _42 property contains the (4,2) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _42 = { read=FGet_42, write=FSet_42, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _43
    // Description: The _43 property contains the (4,3) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _43 = { read=FGet_43, write=FSet_43, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    _44
    // Description: The _44 property contains the (4,4) element of the 
    //              matrix. 
    // ----------------------------------------------------------------------
    __property D3DVALUE _44 = { read=FGet_44, write=FSet_44, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DMatrix's internal D3DMATRIX structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DMatrix(TComponent* Owner); 
    virtual __fastcall ~TD3DMatrix(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DMATRIX* Internal_D3DMATRIX_Ptr = { 
    read=FGetInternal_D3DMATRIX_Ptr, nodefault }; 

    void __fastcall Internal_D3DMATRIX_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    D3DVALUE __fastcall FGet_11(); 
    void __fastcall FSet_11( D3DVALUE p_11 ); 
    D3DVALUE __fastcall FGet_12(); 
    void __fastcall FSet_12( D3DVALUE p_12 ); 
    D3DVALUE __fastcall FGet_13(); 
    void __fastcall FSet_13( D3DVALUE p_13 ); 
    D3DVALUE __fastcall FGet_14(); 
    void __fastcall FSet_14( D3DVALUE p_14 ); 
    D3DVALUE __fastcall FGet_21(); 
    void __fastcall FSet_21( D3DVALUE p_21 ); 
    D3DVALUE __fastcall FGet_22(); 
    void __fastcall FSet_22( D3DVALUE p_22 ); 
    D3DVALUE __fastcall FGet_23(); 
    void __fastcall FSet_23( D3DVALUE p_23 ); 
    D3DVALUE __fastcall FGet_24(); 
    void __fastcall FSet_24( D3DVALUE p_24 ); 
    D3DVALUE __fastcall FGet_31(); 
    void __fastcall FSet_31( D3DVALUE p_31 ); 
    D3DVALUE __fastcall FGet_32(); 
    void __fastcall FSet_32( D3DVALUE p_32 ); 
    D3DVALUE __fastcall FGet_33(); 
    void __fastcall FSet_33( D3DVALUE p_33 ); 
    D3DVALUE __fastcall FGet_34(); 
    void __fastcall FSet_34( D3DVALUE p_34 ); 
    D3DVALUE __fastcall FGet_41(); 
    void __fastcall FSet_41( D3DVALUE p_41 ); 
    D3DVALUE __fastcall FGet_42(); 
    void __fastcall FSet_42( D3DVALUE p_42 ); 
    D3DVALUE __fastcall FGet_43(); 
    void __fastcall FSet_43( D3DVALUE p_43 ); 
    D3DVALUE __fastcall FGet_44(); 
    void __fastcall FSet_44( D3DVALUE p_44 ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DMATRIX* __fastcall FGetInternal_D3DMATRIX_Ptr(); 
    void __fastcall FSetInternal_D3DMATRIX( D3DMATRIX* pD3DMATRIX ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DMATRIX fD3DMATRIX; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
