#ifndef TD3DVertexH
#define TD3DVertexH

// ==========================================================================
// File:         TD3DVertex.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TD3DVertex Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TD3DVertex
// Description: The TD3DVertex component wraps multiple DirectX D3DVERTEX 
//              structures defining utransformed and unlit vertices in terms 
//              of their model coordinates and normal vectors. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TD3DVertex : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TD3DVertex : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TD3DVertex component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              D3DVERTEX structures wrapped inside the TD3DVertex 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TD3DVertex component wraps a single D3DVERTEX 
    //              structure. To wrap multiple structures, set the 
    //              TD3DVertex::ArraySize to the total number of internal 
    //              structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TD3DVertex method or fget/fset. 
    //              eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    NX
    // Description: The NX property contains a D3DVALUE defining the X axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the nx property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE NX[dword pArrayIndex] = { read=FGetNX, write=FSetNX 
    }; 

    // ----------------------------------------------------------------------
    // Property:    NY
    // Description: The NY property contains a D3DVALUE defining the Y axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the ny property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE NY[dword pArrayIndex] = { read=FGetNY, write=FSetNY 
    }; 

    // ----------------------------------------------------------------------
    // Property:    NZ
    // Description: The NZ property contains a D3DVALUE defining the Z axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the nz property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE NZ[dword pArrayIndex] = { read=FGetNZ, write=FSetNZ 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal D3DVERTEX structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal D3DVERTEX structures. This result is the same as 
    //              multiplying TD3DVertex::ArraySize and TD3DVertex::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    TU
    // Description: The TU property contains a D3DVALUE defining the U 
    //              texture coordinate of this vertex. 
    //              
    //              This property is unioned with the tu property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TU[dword pArrayIndex] = { read=FGetTU, write=FSetTU 
    }; 

    // ----------------------------------------------------------------------
    // Property:    TV
    // Description: The TV property contains a D3DVALUE defining the V 
    //              texture coordinate of this vertex. 
    //              
    //              This property is unioned with the tv property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE TV[dword pArrayIndex] = { read=FGetTV, write=FSetTV 
    }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property contains a D3DVALUE defining the 
    //              homogenous X axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the x property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE X[dword pArrayIndex] = { read=FGetX, write=FSetX }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property contains a D3DVALUE defining the 
    //              homogenous Y axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the y property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Y[dword pArrayIndex] = { read=FGetY, write=FSetY }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property contains a D3DVALUE defining the 
    //              homogenous Z axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the z property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE Z[dword pArrayIndex] = { read=FGetZ, write=FSetZ }; 

    // ----------------------------------------------------------------------
    // Property:    nx
    // Description: The nx property contains a D3DVALUE defining the X axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the NX property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE nx[dword pArrayIndex] = { read=FGetnx, write=FSetnx 
    }; 

    // ----------------------------------------------------------------------
    // Property:    ny
    // Description: The ny property contains a D3DVALUE defining the Y axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the NY property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE ny[dword pArrayIndex] = { read=FGetny, write=FSetny 
    }; 

    // ----------------------------------------------------------------------
    // Property:    nz
    // Description: The nz property contains a D3DVALUE defining the Z axis 
    //              coordinate of the normal vector of this vertex. 
    //              
    //              This property is unioned with the NZ property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE nz[dword pArrayIndex] = { read=FGetnz, write=FSetnz 
    }; 

    // ----------------------------------------------------------------------
    // Property:    tu
    // Description: The tu property contains a D3DVALUE defining the U 
    //              texture coordinate of this vertex. 
    //              
    //              This property is unioned with the TU property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tu[dword pArrayIndex] = { read=FGettu, write=FSettu 
    }; 

    // ----------------------------------------------------------------------
    // Property:    tv
    // Description: The tv property contains a D3DVALUE defining the V 
    //              texture coordinate of this vertex. 
    //              
    //              This property is unioned with the TV property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE tv[dword pArrayIndex] = { read=FGettv, write=FSettv 
    }; 

    // ----------------------------------------------------------------------
    // Property:    x
    // Description: The x property contains a D3DVALUE defining the 
    //              homogenous X axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the X property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE x[dword pArrayIndex] = { read=FGetx, write=FSetx }; 

    // ----------------------------------------------------------------------
    // Property:    y
    // Description: The y property contains a D3DVALUE defining the 
    //              homogenous Y axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the Y property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE y[dword pArrayIndex] = { read=FGety, write=FSety }; 

    // ----------------------------------------------------------------------
    // Property:    z
    // Description: The z property contains a D3DVALUE defining the 
    //              homogenous Z axis coordinate. 
    //              
    //              This value is relative to the origin specified for the 
    //              objects model space. 
    //              This property is unioned with the Z property of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property D3DVALUE z[dword pArrayIndex] = { read=FGetz, write=FSetz }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TD3DVertex's internal D3DVERTEX structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TD3DVertex::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal D3DVERTEX structures of the 
    //              TD3DVertex component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TD3DVertex(TComponent* Owner); 
    virtual __fastcall ~TD3DVertex(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property D3DVERTEX* Internal_D3DVERTEX_Ptr = { 
    read=FGetInternal_D3DVERTEX_Ptr, nodefault }; 

    void __fastcall Internal_D3DVERTEX_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    D3DVALUE __fastcall FGetNX( dword pArrayIndex ); 
    void __fastcall FSetNX( dword pArrayIndex, D3DVALUE pNX ); 
    D3DVALUE __fastcall FGetNY( dword pArrayIndex ); 
    void __fastcall FSetNY( dword pArrayIndex, D3DVALUE pNY ); 
    D3DVALUE __fastcall FGetNZ( dword pArrayIndex ); 
    void __fastcall FSetNZ( dword pArrayIndex, D3DVALUE pNZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    D3DVALUE __fastcall FGetTU( dword pArrayIndex ); 
    void __fastcall FSetTU( dword pArrayIndex, D3DVALUE pTU ); 
    D3DVALUE __fastcall FGetTV( dword pArrayIndex ); 
    void __fastcall FSetTV( dword pArrayIndex, D3DVALUE pTV ); 
    D3DVALUE __fastcall FGetX( dword pArrayIndex ); 
    void __fastcall FSetX( dword pArrayIndex, D3DVALUE pX ); 
    D3DVALUE __fastcall FGetY( dword pArrayIndex ); 
    void __fastcall FSetY( dword pArrayIndex, D3DVALUE pY ); 
    D3DVALUE __fastcall FGetZ( dword pArrayIndex ); 
    void __fastcall FSetZ( dword pArrayIndex, D3DVALUE pZ ); 
    D3DVALUE __fastcall FGetnx( dword pArrayIndex ); 
    void __fastcall FSetnx( dword pArrayIndex, D3DVALUE pnx ); 
    D3DVALUE __fastcall FGetny( dword pArrayIndex ); 
    void __fastcall FSetny( dword pArrayIndex, D3DVALUE pny ); 
    D3DVALUE __fastcall FGetnz( dword pArrayIndex ); 
    void __fastcall FSetnz( dword pArrayIndex, D3DVALUE pnz ); 
    D3DVALUE __fastcall FGettu( dword pArrayIndex ); 
    void __fastcall FSettu( dword pArrayIndex, D3DVALUE ptu ); 
    D3DVALUE __fastcall FGettv( dword pArrayIndex ); 
    void __fastcall FSettv( dword pArrayIndex, D3DVALUE ptv ); 
    D3DVALUE __fastcall FGetx( dword pArrayIndex ); 
    void __fastcall FSetx( dword pArrayIndex, D3DVALUE px ); 
    D3DVALUE __fastcall FGety( dword pArrayIndex ); 
    void __fastcall FSety( dword pArrayIndex, D3DVALUE py ); 
    D3DVALUE __fastcall FGetz( dword pArrayIndex ); 
    void __fastcall FSetz( dword pArrayIndex, D3DVALUE pz ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    D3DVERTEX* __fastcall FGetInternal_D3DVERTEX_Ptr(); 
    void __fastcall FSetInternal_D3DVERTEX( dword pArrayIndex, D3DVERTEX* 
    pD3DVERTEX ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    D3DVERTEX* fD3DVERTEX; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
