#ifndef TDDColorControlH
#define TDDColorControlH

// ==========================================================================
// File:         TDDColorControl.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDColorControl Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// ==========================================================================
// Set:         TDDColorControlFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDColorControl::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDColorControl_Flags_Flags
    {
    ddcolor_brightness,
    ddcolor_colorenable,
    ddcolor_contrast,
    ddcolor_gamma,
    ddcolor_hue,
    ddcolor_saturation,
    ddcolor_sharpness
    };
typedef Set< TDDColorControl_Flags_Flags, 
             ddcolor_brightness, 
             ddcolor_sharpness >  TDDColorControl_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDColorControl
// Description: The TDDColorControl component wraps the DirectX 
//              DDCOLORCONTROL structure, which defines the color controls 
//              associated with a video port, overlay surface or primary 
//              surface. 
//              
//              It is used by TDx_DrawColorControl::GetColorControls() and 
//              TDx_DrawColorControl::SetColorControls() as well as by 
//              TDx_DrawVideoPort::GetColorControls() and 
//              TDx_DrawVideoPort::SetColorControls() to retrieve or define 
//              color control settings. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDColorControl : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDColorControl : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating which of this 
    //              components properties contain valid data. 
    //              When returned by 
    //              TDx_DrawColorControl::GetColorControls(), these flags 
    //              indicate which color controls are supported. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDColorControl_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Brightness
    // Description: The Brightness property defines the luminance intensity, 
    //              in IRE units, multiplied by 100. 
    //              The possible range is from 0 to 10,000 with a default of 
    //              750. 
    // ----------------------------------------------------------------------
    __property long Brightness = { read=FGetBrightness, write=FSetBrightness, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ColorEnable
    // Description: The ColorEnable property defines whether color is 
    //              utilized or not. 
    //              Color is used if this property is 1. Color is not used if 
    //              this property is 0. The default value is 1. 
    // ----------------------------------------------------------------------
    __property long ColorEnable = { read=FGetColorEnable, 
    write=FSetColorEnable, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Contrast
    // Description: The Contrast property defines the relative difference 
    //              between higher and lower luminance values, in IRE units, 
    //              multiplied by 100. 
    //              The possible range is from 0 to 20,000 with a default 
    //              value of 10,000. 
    // ----------------------------------------------------------------------
    __property long Contrast = { read=FGetContrast, write=FSetContrast, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Gamma
    // Description: The Gamma property defines the amount, in gamma units, of 
    //              gamma correction applied to the luminance values. 
    //              The possible range is from 1 to 500, with a default of 1. 
    // ----------------------------------------------------------------------
    __property long Gamma = { read=FGetGamma, write=FSetGamma, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Hue
    // Description: The Hue property defines the phase relationship, in 
    //              degrees, of the chrominance components. 
    //              The possible range is from -180 to 180, with a default of 
    //              0. 
    // ----------------------------------------------------------------------
    __property long Hue = { read=FGetHue, write=FSetHue, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Saturation
    // Description: The Saturation property defines the color intensity, in 
    //              IRE units, multiplied by 100. 
    //              The possible range is 0 to 20,000, with a default value 
    //              of 10,000. 
    // ----------------------------------------------------------------------
    __property long Saturation = { read=FGetSaturation, write=FSetSaturation, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Sharpness
    // Description: The Sharpness property defines the sharpness, in 
    //              arbitrary units, of an image. 
    //              The possible range is 0 to 10, with a default value of 5. 
    // ----------------------------------------------------------------------
    __property long Sharpness = { read=FGetSharpness, write=FSetSharpness, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDCOLORCONTROL) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDColorControl component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDColorControl method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating which of this 
    //              components properties contain valid data. 
    //              When returned by 
    //              TDx_DrawColorControl::GetColorControls(), these flags 
    //              indicate which color controls are supported. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDCOLOR_BRIGHTNESS -
    //                TDDColorControl::Brightness contains valid data or 
    //                brightness control is supported. 
    //              DDCOLOR_COLORENABLE -
    //                TDDColorControl::ColorEnable contains valid data or 
    //                color usage can be enabled or disabled. 
    //              DDCOLOR_CONTRAST -
    //                TDDColorControl::Contrast contains valid data or 
    //                contrast control is supported. 
    //              DDCOLOR_GAMMA -
    //                TDDColorControl::Gamma contains valid data or gamma 
    //                control is supported. 
    //              DDCOLOR_HUE -
    //                TDDColorControl::Hue contains valid data or hue control 
    //                is supported. 
    //              DDCOLOR_SATURATION -
    //                TDDColorControl::Saturation contains valid data or 
    //                saturation control is supported. 
    //              DDCOLOR_SHARPNESS -
    //                TDDColorControl::Sharpness contains valid data or 
    //                sharpness control is supported. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDColorControl's internal DDCOLORCONTROL structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDColorControl(TComponent* Owner); 
    virtual __fastcall ~TDDColorControl(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDCOLORCONTROL* Internal_DDCOLORCONTROL_Ptr = { 
    read=FGetInternal_DDCOLORCONTROL_Ptr, nodefault }; 

    void __fastcall Internal_DDCOLORCONTROL_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDColorControl_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDDColorControl_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    long __fastcall FGetBrightness(); 
    void __fastcall FSetBrightness( long pBrightness ); 
    long __fastcall FGetColorEnable(); 
    void __fastcall FSetColorEnable( long pColorEnable ); 
    long __fastcall FGetContrast(); 
    void __fastcall FSetContrast( long pContrast ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    long __fastcall FGetGamma(); 
    void __fastcall FSetGamma( long pGamma ); 
    long __fastcall FGetHue(); 
    void __fastcall FSetHue( long pHue ); 
    long __fastcall FGetSaturation(); 
    void __fastcall FSetSaturation( long pSaturation ); 
    long __fastcall FGetSharpness(); 
    void __fastcall FSetSharpness( long pSharpness ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDCOLORCONTROL* __fastcall FGetInternal_DDCOLORCONTROL_Ptr(); 
    void __fastcall FSetInternal_DDCOLORCONTROL( DDCOLORCONTROL* 
    pDDCOLORCONTROL ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDCOLORCONTROL fDDCOLORCONTROL; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
