#ifndef TDDOverlayFXH
#define TDDOverlayFXH

// ==========================================================================
// File:         TDDOverlayFX.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDOverlayFX Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDDOverlayFX properties.
#include "TDDColorKey.H"
#include "TDx_DrawSurface.H"

// ==========================================================================
// Set:         TDDOverlayFXFX_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDOverlayFX::FX property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the FX property 
//              description below. 
// ==========================================================================
enum TDDOverlayFX_FX_Flags
    {
    ddoverfx_arithstretchy,
    ddoverfx_mirrorleftright,
    ddoverfx_mirrorupdown
    };
typedef Set< TDDOverlayFX_FX_Flags, 
             ddoverfx_arithstretchy, 
             ddoverfx_mirrorupdown >  TDDOverlayFX_FX_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDOverlayFX
// Description: The TDDOverlayFX component wraps the DirectX DDOVERLAYFX 
//              structure, which describes modifications to the execution of 
//              overlay operations. 
//              
//              It is used by the TDx_DrawSurface::UpdateOverlay() method to 
//              specify raster operations, effects and to provide override 
//              information when the corresponding flags are selected. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDOverlayFX : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDOverlayFX : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    FX_
    // Description: The FX property defines flags indicating what overlay 
    //              stretching or mirroring effects are to be used. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'FX' property via the BCB Object Inspector.  For 
    //              detailed descriptions of each flag, see the FX property 
    //              description below. 
    // ----------------------------------------------------------------------
    __property TDDOverlayFX_FX_Set FX_ = { read=FGetFX_, write=FSetFX_, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaDestConst
    // Description: The AlphaDestConst property defines the constant used as 
    //              the destination overlay alpha channel when 
    //              DDOVER_ALPHADESTCONSTOVERRIDE is set in the Flags 
    //              parameter of TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property dword AlphaDestConst = { read=FGetAlphaDestConst, 
    write=FSetAlphaDestConst, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaDestConstBitDepth
    // Description: The AlphaDestConstBitDepth property defines the bit depth 
    //              of TDDOverlayFX::AlphaDestConst. 
    // ----------------------------------------------------------------------
    __property dword AlphaDestConstBitDepth = { 
    read=FGetAlphaDestConstBitDepth, write=FSetAlphaDestConstBitDepth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaEdgeBlend
    // Description: The AlphaEdgeBlend property defines the constant to be 
    //              used for alpha edge blending when DDOVER_ALPHAEDGEBLEND 
    //              is set in the Flags parameter of 
    //              TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property dword AlphaEdgeBlend = { read=FGetAlphaEdgeBlend, 
    write=FSetAlphaEdgeBlend, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaEdgeBlendBitDepth
    // Description: The AlphaEdgeBlendBitDepth property defines the bit depth 
    //              of TDDOverlayFX::AlphaEdgeBlend. 
    // ----------------------------------------------------------------------
    __property dword AlphaEdgeBlendBitDepth = { 
    read=FGetAlphaEdgeBlendBitDepth, write=FSetAlphaEdgeBlendBitDepth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaSrcConst
    // Description: The AlphaSrcConst property defines the constant used as 
    //              the source overlay alpha channel when 
    //              DDOVER_ALPHASRCCONSTOVERRIDE is set in the Flags 
    //              parameter of TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property dword AlphaSrcConst = { read=FGetAlphaSrcConst, 
    write=FSetAlphaSrcConst, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaSrcConstBitDepth
    // Description: The AlphaSrcConstBitDepth property defines the bit depth 
    //              of TDDOverlayFX::AlphaSrcConst. 
    // ----------------------------------------------------------------------
    __property dword AlphaSrcConstBitDepth = { 
    read=FGetAlphaSrcConstBitDepth, write=FSetAlphaSrcConstBitDepth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    DestColorkey
    // Description: The DestColorkey property references the destination 
    //              color key to be used when DDOVER_KEYDESTOVERRIDE flag is 
    //              set in the Flags parameter of 
    //              TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property TDDColorKey* DestColorkey = { read=FGetDestColorkey, 
    write=FSetDestColorkey, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    SAlphaDest
    // Description: The SAlphaDest property references the surface used as 
    //              the destination overlay alpha channel when 
    //              DDOVER_ALPHADESTSURFACEOVERRIDE is set in the Flags 
    //              parameter of TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property TDx_DrawSurface* SAlphaDest = { read=FGetSAlphaDest, 
    write=FSetSAlphaDest, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    SAlphaSrc
    // Description: The SAlphaSrc property references the surface used as the 
    //              source overlay alpha channel when 
    //              DDOVER_ALPHASRCSURFACEOVERRIDE flag is set in the Flags 
    //              parameter of TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property TDx_DrawSurface* SAlphaSrc = { read=FGetSAlphaSrc, 
    write=FSetSAlphaSrc, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDOVERLAYFX) }; 

    // ----------------------------------------------------------------------
    // Property:    SrcColorkey
    // Description: The SrcColorkey property references the source color key 
    //              to be used when DDOVER_KEYSRCOVERRIDE is set in the Flags 
    //              parameter of TDx_DrawSurface::UpdateOverlay(). 
    // ----------------------------------------------------------------------
    __property TDDColorKey* SrcColorkey = { read=FGetSrcColorkey, 
    write=FSetSrcColorkey, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDOverlayFX component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDOverlayFX method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    FX
    // Description: The FX property defines flags indicating what overlay 
    //              stretching or mirroring effects are to be used. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDOVERFX_ARITHSTRETCHY -
    //                Stretching of the overlay along the Y axis will utilize 
    //                arithmetic stretching techniques. 
    //              DDOVERFX_MIRRORLEFTRIGHT -
    //                The overlay is to be mirrored around the Y axis. 
    //              DDOVERFX_MIRRORUPDOWN -
    //                The overlay is to be mirrored around the X axis. 
    // ----------------------------------------------------------------------
    __property dword FX = { read=FGetFX, write=FSetFX, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDOverlayFX's internal DDOVERLAYFX structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    FX_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the FX property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* FX_Strings = { read=FGetFX_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDOverlayFX(TComponent* Owner); 
    virtual __fastcall ~TDDOverlayFX(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDOVERLAYFX* Internal_DDOVERLAYFX_Ptr = { 
    read=FGetInternal_DDOVERLAYFX_Ptr, nodefault }; 

    void __fastcall Internal_DDOVERLAYFX_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDOverlayFX_FX_Set __fastcall FGetFX_(); 
    void __fastcall FSetFX_( TDDOverlayFX_FX_Set pFX_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetAlphaDestConst(); 
    void __fastcall FSetAlphaDestConst( dword pAlphaDestConst ); 
    dword __fastcall FGetAlphaDestConstBitDepth(); 
    void __fastcall FSetAlphaDestConstBitDepth( dword pAlphaDestConstBitDepth 
    ); 
    dword __fastcall FGetAlphaEdgeBlend(); 
    void __fastcall FSetAlphaEdgeBlend( dword pAlphaEdgeBlend ); 
    dword __fastcall FGetAlphaEdgeBlendBitDepth(); 
    void __fastcall FSetAlphaEdgeBlendBitDepth( dword pAlphaEdgeBlendBitDepth 
    ); 
    dword __fastcall FGetAlphaSrcConst(); 
    void __fastcall FSetAlphaSrcConst( dword pAlphaSrcConst ); 
    dword __fastcall FGetAlphaSrcConstBitDepth(); 
    void __fastcall FSetAlphaSrcConstBitDepth( dword pAlphaSrcConstBitDepth 
    ); 
    TDDColorKey* __fastcall FGetDestColorkey(); 
    void __fastcall FSetDestColorkey( TDDColorKey* pDestColorkey ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFX(); 
    void __fastcall FSetFX( dword pFX ); 
    TDx_DrawSurface* __fastcall FGetSAlphaDest(); 
    void __fastcall FSetSAlphaDest( TDx_DrawSurface* pSAlphaDest ); 
    TDx_DrawSurface* __fastcall FGetSAlphaSrc(); 
    void __fastcall FSetSAlphaSrc( TDx_DrawSurface* pSAlphaSrc ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    TDDColorKey* __fastcall FGetSrcColorkey(); 
    void __fastcall FSetSrcColorkey( TDDColorKey* pSrcColorkey ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFX_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDOVERLAYFX* __fastcall FGetInternal_DDOVERLAYFX_Ptr(); 
    void __fastcall FSetInternal_DDOVERLAYFX( DDOVERLAYFX* pDDOVERLAYFX ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    TDDColorKey* fDestColorkey; 
    HRESULT fErrorValue; 
    TDx_DrawSurface* fSAlphaDest; 
    TDx_DrawSurface* fSAlphaSrc; 
    TDDColorKey* fSrcColorkey; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFX_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDOVERLAYFX fDDOVERLAYFX; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
