#ifndef TDDPixelFormatH
#define TDDPixelFormatH

// ==========================================================================
// File:         TDDPixelFormat.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDPixelFormat Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// ==========================================================================
// Set:         TDDPixelFormatFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDPixelFormat::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDPixelFormat_Flags_Flags
    {
    ddpf_alpha,
    ddpf_alphapixels,
    ddpf_alphapremult,
    ddpf_bumpdudv,
    ddpf_bumpluminance,
    ddpf_compressed,
    ddpf_fourcc,
    ddpf_luminance,
    ddpf_paletteindexed1,
    ddpf_paletteindexed2,
    ddpf_paletteindexed4,
    ddpf_paletteindexed8,
    ddpf_paletteindexedto8,
    ddpf_rgb,
    ddpf_rgbtoyuv,
    ddpf_stencilbuffer,
    ddpf_yuv,
    ddpf_zbuffer,
    ddpf_zpixels
    };
typedef Set< TDDPixelFormat_Flags_Flags, 
             ddpf_alpha, 
             ddpf_zpixels >  TDDPixelFormat_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDPixelFormat
// Description: The TDDPixelFormat component wraps multiple DDPIXELFORMAT 
//              structures, which describe the pixel format and color 
//              information of a surface. 
//              
//              It is used by TDx_DrawSurface::GetPixelFormat(), 
//              TDx_DrawVideoPort::GetBandwidthInfo(), 
//              TDx_DrawVideoPort::GetInputFormats and 
//              TDx_DrawVideoPort::GetOutputFormats when retrieving color and 
//              pixel format information. 
//              It is also present in TDDSurfaceDesc and TDDVideoPortInfo. 
//              
//              To setup this component, first set the 
//              TDDPixelFormat::ArraySize property, then set each property 
//              using array indexes, eg: 
//              
//                  DDPixelFormat1->ArraySize = 1; 
//                  DDPixelFormat1->Flags[0] = DDPF_PALETTEINDEXED8; 
//              
//              TDDPixelFormat::AlphaBitDepth refers to the bit depth for 
//              alpha-only pixel formats (DDPF_ALPHA). 
//              To obtain the alpha bit depth when the alpa component is 
//              included with color components (DDPF_ALPHAPIXELS), count the 
//              bits in the various mask members. 
//              For example, this function will return the number of bits set 
//              in a given bitmask: 
//              
//              WORD GetNumberOfBits( DWORD Mask ) 
//              { 
//                  WORD Bits = 0; 
//                  while( Mask ) 
//                  { 
//                      Mask = Mask & ( Mask - 1 );   
//                      Bits++; 
//                  } 
//                  return Bits; 
//              } 
//              
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDPixelFormat : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDPixelFormat : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDPixelFormat component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    AlphaBitDepth
    // Description: The AlphaBitDepth property defines a value of 1, 2, 4 or 
    //              8 representing the bit depth of an alpha only surface. 
    // ----------------------------------------------------------------------
    __property dword AlphaBitDepth[dword pArrayIndex] = { 
    read=FGetAlphaBitDepth, write=FSetAlphaBitDepth }; 

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DDPIXELFORMAT structures wrapped inside the 
    //              TDDPixelFormat component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDDPixelFormat component wraps a single 
    //              DDPIXELFORMAT structure. To wrap multiple structures, set 
    //              the TDDPixelFormat::ArraySize to the total number of 
    //              internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    BBitMask
    // Description: The BBitMask property defines the mask for handling the 
    //              blue component of a RGB surface. 
    // ----------------------------------------------------------------------
    __property dword BBitMask[dword pArrayIndex] = { read=FGetBBitMask, 
    write=FSetBBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    BumpBitCount
    // Description: The BumpBitCount property defines the number of bump map 
    //              bits per pixel in a bump map surface. 
    // ----------------------------------------------------------------------
    __property dword BumpBitCount[dword pArrayIndex] = { 
    read=FGetBumpBitCount, write=FSetBumpBitCount }; 

    // ----------------------------------------------------------------------
    // Property:    BumpDuBitMask
    // Description: The BumpDuBitMask property defines the bit mask for 
    //              handling the bump map delta U component of a pixel. 
    // ----------------------------------------------------------------------
    __property dword BumpDuBitMask[dword pArrayIndex] = { 
    read=FGetBumpDuBitMask, write=FSetBumpDuBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    BumpDvBitMask
    // Description: The BumpDvBitMask property defines the bit mask for 
    //              handling the bump map delta V component of a pixel. 
    // ----------------------------------------------------------------------
    __property dword BumpDvBitMask[dword pArrayIndex] = { 
    read=FGetBumpDvBitMask, write=FSetBumpDvBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    BumpLuminanceBitMask
    // Description: The BumpLuminanceBitMask property defines the bit mask 
    //              for handling the bump map luminance component of a pixel. 
    // ----------------------------------------------------------------------
    __property dword BumpLuminanceBitMask[dword pArrayIndex] = { 
    read=FGetBumpLuminanceBitMask, write=FSetBumpLuminanceBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDPixelFormat method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating various 
    //              control options and which properties of this component 
    //              contain valid data. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDPF_ALPHA -
    //                This component is describing the pixel format of an 
    //                alpha only surface. 
    //              DDPF_ALPHAPIXELS -
    //                This component is describing the pixel format of a 
    //                surface with interleaved alpha channel information. 
    //              DDPF_ALPHAPREMULT -
    //                This component is describing the pixel format of a 
    //                surface using a premultiplied alpha format. 
    //                The alpha component is premultiplied with the color 
    //                components for each pixel. 
    //              DDPF_BUMPDUDV -
    //                The bump map related properties of this component 
    //                contain valid data. 
    //              DDPF_BUMPLUMINANCE -
    //                The luminance related properties of this component 
    //                contain valid data. 
    //              DDPF_COMPRESSED -
    //                The surface this component describes will accept pixel 
    //                data in a specified format and compress it during write 
    //                operations. 
    //              DDPF_FOURCC -
    //                The FourCC property of this component contains valid 
    //                data. 
    //              DDPF_LUMINANCE -
    //                This component is describing the pixel format of a 
    //                luminance or luminance alpha surface. 
    //              DDPF_PALETTEINDEXED1 -
    //                This component is describing the pixel format of a 1 
    //                bit color palette indexed surface. 
    //              DDPF_PALETTEINDEXED2 -
    //                This component is describing the pixel format of a 2 
    //                bit color palette indexed surface. 
    //              DDPF_PALETTEINDEXED4 -
    //                This component is describing the pixel format of a 4 
    //                bit color palette indexed surface. 
    //              DDPF_PALETTEINDEXED8 -
    //                This component is describing the pixel format of a 8 
    //                bit color palette indexed surface. 
    //              DDPF_PALETTEINDEXEDTO8 -
    //                This component is describing the pixel format of a 
    //                surface that uses a 1, 2 or 4 bit index to an 8 bit 
    //                color palette. 
    //              DDPF_RGB -
    //                The RGB properties of this component contain valid 
    //                data. 
    //              DDPF_RGBTOYUV -
    //                The surface this component is describing will accept 
    //                RGB data and translate it to YUV data during the write 
    //                operation. 
    //                The format of the data being written is contained in 
    //                the pixel format. 
    //                DDPF_RGB will also be set. 
    //              DDPF_STENCILBUFFER -
    //                This component is describing the pixel format of a 
    //                z-buffer surface which also encodes stencil 
    //                information. 
    //                This flag may only be utilized when DDPF_ZBUFFER is 
    //                also set. 
    //              DDPF_YUV -
    //                The YUV properties in this component contain valid 
    //                data.. 
    //              DDPF_ZBUFFER -
    //                This component is describing the pixel format of a 
    //                z-buffer only surface. 
    //              DDPF_ZPIXELS -
    //                This component is describing the pixel format of a 
    //                surface with interleaved z-buffer information. 
    // ----------------------------------------------------------------------
    __property dword Flags[dword pArrayIndex] = { read=FGetFlags, 
    write=FSetFlags }; 

    // ----------------------------------------------------------------------
    // Property:    FourCC
    // Description: The FourCC property defines a FourCC code describing the 
    //              data format of the surface this component is describing. 
    // ----------------------------------------------------------------------
    __property dword FourCC[dword pArrayIndex] = { read=FGetFourCC, 
    write=FSetFourCC }; 

    // ----------------------------------------------------------------------
    // Property:    GBitMask
    // Description: The GBitMask property defines the mask for handling the 
    //              green component of a RGB surface. 
    // ----------------------------------------------------------------------
    __property dword GBitMask[dword pArrayIndex] = { read=FGetGBitMask, 
    write=FSetGBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    LuminanceAlphaBitMask
    // Description: The LuminanceAlphaBitMask property defines the bit mask 
    //              for handling the luminance alpha component of a pixel. 
    // ----------------------------------------------------------------------
    __property dword LuminanceAlphaBitMask[dword pArrayIndex] = { 
    read=FGetLuminanceAlphaBitMask, write=FSetLuminanceAlphaBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    LuminanceBitCount
    // Description: The LuminanceBitCount property defines the number of 
    //              luminance bits per pixel in a luminance or luminance 
    //              alpha surface. 
    // ----------------------------------------------------------------------
    __property dword LuminanceBitCount[dword pArrayIndex] = { 
    read=FGetLuminanceBitCount, write=FSetLuminanceBitCount }; 

    // ----------------------------------------------------------------------
    // Property:    LuminanceBitMask
    // Description: The LuminanceBitMask property defines the mask for 
    //              handling the luminance component of a pixel. 
    // ----------------------------------------------------------------------
    __property dword LuminanceBitMask[dword pArrayIndex] = { 
    read=FGetLuminanceBitMask, write=FSetLuminanceBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    RBitMask
    // Description: The RBitMask property defines the mask for handling the 
    //              red component of a RGB surface. 
    // ----------------------------------------------------------------------
    __property dword RBitMask[dword pArrayIndex] = { read=FGetRBitMask, 
    write=FSetRBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    RGBAlphaBitMask
    // Description: The RGBAlphaBitMask property defines the mask for 
    //              handling the alpha channel of a RGBA surface. 
    // ----------------------------------------------------------------------
    __property dword RGBAlphaBitMask[dword pArrayIndex] = { 
    read=FGetRGBAlphaBitMask, write=FSetRGBAlphaBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    RGBBitCount
    // Description: The RGBBitCount property defines a value of 4, 8, 16, 24 
    //              or 32 representing the number of bits per pixel for a RGB 
    //              surface. 
    // ----------------------------------------------------------------------
    __property dword RGBBitCount[dword pArrayIndex] = { read=FGetRGBBitCount, 
    write=FSetRGBBitCount }; 

    // ----------------------------------------------------------------------
    // Property:    RGBZBitMask
    // Description: The RGBZBitMask property defines the mask for handling 
    //              the z-buffer component of a RGBZ surface. 
    // ----------------------------------------------------------------------
    __property dword RGBZBitMask[dword pArrayIndex] = { read=FGetRGBZBitMask, 
    write=FSetRGBZBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size[dword pArrayIndex] = { read=FGetSize, 
    write=FSetSize }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DDPIXELFORMAT structures. This result is the 
    //              same as multiplying TDDPixelFormat::ArraySize and 
    //              TDDPixelFormat::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    StencilBitDepth
    // Description: The StencilBitCount property defines the number of 
    //              stencil bits per pixel in a z-buffer surface. 
    //              The actual z-buffer bit depth can be determined by 
    //              subtracting this property from 
    //              TDDPixelFormat::ZBufferBitDepth. 
    // ----------------------------------------------------------------------
    __property dword StencilBitDepth[dword pArrayIndex] = { 
    read=FGetStencilBitDepth, write=FSetStencilBitDepth }; 

    // ----------------------------------------------------------------------
    // Property:    StencilBitMask
    // Description: The StencilBitMask property defines the bit mask for 
    //              handling the stencil component of a z-buffer. 
    // ----------------------------------------------------------------------
    __property dword StencilBitMask[dword pArrayIndex] = { 
    read=FGetStencilBitMask, write=FSetStencilBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    UBitMask
    // Description: The UBitMask property defines the mask for handling the U 
    //              component of a YUV surface. 
    // ----------------------------------------------------------------------
    __property dword UBitMask[dword pArrayIndex] = { read=FGetUBitMask, 
    write=FSetUBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    VBitMask
    // Description: The VBitMask property defines the mask for handling the V 
    //              component of a YUV surface. 
    // ----------------------------------------------------------------------
    __property dword VBitMask[dword pArrayIndex] = { read=FGetVBitMask, 
    write=FSetVBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    YBitMask
    // Description: The YBitMask property defines the mask for handling the Y 
    //              component of a YUV surface. 
    // ----------------------------------------------------------------------
    __property dword YBitMask[dword pArrayIndex] = { read=FGetYBitMask, 
    write=FSetYBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    YUVAlphaBitMask
    // Description: The YUVAlphaBitMask property defines the mask for 
    //              handling the alpha channel component of a YUVA surface. 
    // ----------------------------------------------------------------------
    __property dword YUVAlphaBitMask[dword pArrayIndex] = { 
    read=FGetYUVAlphaBitMask, write=FSetYUVAlphaBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    YUVBitCount
    // Description: The YUVBitCount property defines a value of 4, 8, 16, 24 
    //              or 32 representing the number of bits per pixel for a YUV 
    //              surface. 
    // ----------------------------------------------------------------------
    __property dword YUVBitCount[dword pArrayIndex] = { read=FGetYUVBitCount, 
    write=FSetYUVBitCount }; 

    // ----------------------------------------------------------------------
    // Property:    YUVZBitMask
    // Description: The YUVZBitMask property defines the mask for handling 
    //              the z-buffer component of a YUVZ surface. 
    // ----------------------------------------------------------------------
    __property dword YUVZBitMask[dword pArrayIndex] = { read=FGetYUVZBitMask, 
    write=FSetYUVZBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    ZBitMask
    // Description: The ZBitMask property defines the bit mask for handling 
    //              the z-value component of a z-buffer. 
    // ----------------------------------------------------------------------
    __property dword ZBitMask[dword pArrayIndex] = { read=FGetZBitMask, 
    write=FSetZBitMask }; 

    // ----------------------------------------------------------------------
    // Property:    ZBufferBitDepth
    // Description: The ZBufferBitDepth property defines a value of 8, 16, 24 
    //              or 32 representing the bit depth of the z-buffer. 
    // ----------------------------------------------------------------------
    __property dword ZBufferBitDepth[dword pArrayIndex] = { 
    read=FGetZBufferBitDepth, write=FSetZBufferBitDepth }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDPixelFormat's internal DDPIXELFORMAT structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDDPixelFormat::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DDPIXELFORMAT structures of the 
    //              TDDPixelFormat component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings[dword pArrayIndex] = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating various 
    //              control options and which properties of this component 
    //              contain valid data. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDPixelFormat_Flags_Set Flags_[dword pArrayIndex] = { 
    read=FGetFlags_, write=FSetFlags_ }; 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDPixelFormat(TComponent* Owner); 
    virtual __fastcall ~TDDPixelFormat(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDPIXELFORMAT* Internal_DDPIXELFORMAT_Ptr = { 
    read=FGetInternal_DDPIXELFORMAT_Ptr, nodefault }; 

    void __fastcall Internal_DDPIXELFORMAT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDPixelFormat_Flags_Set __fastcall FGetFlags_( dword pArrayIndex ); 
    void __fastcall FSetFlags_( dword pArrayIndex, TDDPixelFormat_Flags_Set 
    pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetAlphaBitDepth( dword pArrayIndex ); 
    void __fastcall FSetAlphaBitDepth( dword pArrayIndex, dword 
    pAlphaBitDepth ); 
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    dword __fastcall FGetBBitMask( dword pArrayIndex ); 
    void __fastcall FSetBBitMask( dword pArrayIndex, dword pBBitMask ); 
    dword __fastcall FGetBumpBitCount( dword pArrayIndex ); 
    void __fastcall FSetBumpBitCount( dword pArrayIndex, dword pBumpBitCount 
    ); 
    dword __fastcall FGetBumpDuBitMask( dword pArrayIndex ); 
    void __fastcall FSetBumpDuBitMask( dword pArrayIndex, dword 
    pBumpDuBitMask ); 
    dword __fastcall FGetBumpDvBitMask( dword pArrayIndex ); 
    void __fastcall FSetBumpDvBitMask( dword pArrayIndex, dword 
    pBumpDvBitMask ); 
    dword __fastcall FGetBumpLuminanceBitMask( dword pArrayIndex ); 
    void __fastcall FSetBumpLuminanceBitMask( dword pArrayIndex, dword 
    pBumpLuminanceBitMask ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags( dword pArrayIndex ); 
    void __fastcall FSetFlags( dword pArrayIndex, dword pFlags ); 
    dword __fastcall FGetFourCC( dword pArrayIndex ); 
    void __fastcall FSetFourCC( dword pArrayIndex, dword pFourCC ); 
    dword __fastcall FGetGBitMask( dword pArrayIndex ); 
    void __fastcall FSetGBitMask( dword pArrayIndex, dword pGBitMask ); 
    dword __fastcall FGetLuminanceAlphaBitMask( dword pArrayIndex ); 
    void __fastcall FSetLuminanceAlphaBitMask( dword pArrayIndex, dword 
    pLuminanceAlphaBitMask ); 
    dword __fastcall FGetLuminanceBitCount( dword pArrayIndex ); 
    void __fastcall FSetLuminanceBitCount( dword pArrayIndex, dword 
    pLuminanceBitCount ); 
    dword __fastcall FGetLuminanceBitMask( dword pArrayIndex ); 
    void __fastcall FSetLuminanceBitMask( dword pArrayIndex, dword 
    pLuminanceBitMask ); 
    dword __fastcall FGetRBitMask( dword pArrayIndex ); 
    void __fastcall FSetRBitMask( dword pArrayIndex, dword pRBitMask ); 
    dword __fastcall FGetRGBAlphaBitMask( dword pArrayIndex ); 
    void __fastcall FSetRGBAlphaBitMask( dword pArrayIndex, dword 
    pRGBAlphaBitMask ); 
    dword __fastcall FGetRGBBitCount( dword pArrayIndex ); 
    void __fastcall FSetRGBBitCount( dword pArrayIndex, dword pRGBBitCount ); 
    dword __fastcall FGetRGBZBitMask( dword pArrayIndex ); 
    void __fastcall FSetRGBZBitMask( dword pArrayIndex, dword pRGBZBitMask ); 
    dword __fastcall FGetSize( dword pArrayIndex ); 
    void __fastcall FSetSize( dword pArrayIndex, dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    dword __fastcall FGetStencilBitDepth( dword pArrayIndex ); 
    void __fastcall FSetStencilBitDepth( dword pArrayIndex, dword 
    pStencilBitDepth ); 
    dword __fastcall FGetStencilBitMask( dword pArrayIndex ); 
    void __fastcall FSetStencilBitMask( dword pArrayIndex, dword 
    pStencilBitMask ); 
    dword __fastcall FGetUBitMask( dword pArrayIndex ); 
    void __fastcall FSetUBitMask( dword pArrayIndex, dword pUBitMask ); 
    dword __fastcall FGetVBitMask( dword pArrayIndex ); 
    void __fastcall FSetVBitMask( dword pArrayIndex, dword pVBitMask ); 
    dword __fastcall FGetYBitMask( dword pArrayIndex ); 
    void __fastcall FSetYBitMask( dword pArrayIndex, dword pYBitMask ); 
    dword __fastcall FGetYUVAlphaBitMask( dword pArrayIndex ); 
    void __fastcall FSetYUVAlphaBitMask( dword pArrayIndex, dword 
    pYUVAlphaBitMask ); 
    dword __fastcall FGetYUVBitCount( dword pArrayIndex ); 
    void __fastcall FSetYUVBitCount( dword pArrayIndex, dword pYUVBitCount ); 
    dword __fastcall FGetYUVZBitMask( dword pArrayIndex ); 
    void __fastcall FSetYUVZBitMask( dword pArrayIndex, dword pYUVZBitMask ); 
    dword __fastcall FGetZBitMask( dword pArrayIndex ); 
    void __fastcall FSetZBitMask( dword pArrayIndex, dword pZBitMask ); 
    dword __fastcall FGetZBufferBitDepth( dword pArrayIndex ); 
    void __fastcall FSetZBufferBitDepth( dword pArrayIndex, dword 
    pZBufferBitDepth ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings( dword pArrayIndex );

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDPIXELFORMAT* __fastcall FGetInternal_DDPIXELFORMAT_Ptr(); 
    void __fastcall FSetInternal_DDPIXELFORMAT( dword pArrayIndex, 
    DDPIXELFORMAT* pDDPIXELFORMAT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDPIXELFORMAT* fDDPIXELFORMAT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
