#ifndef TDDVideoPortCapsH
#define TDDVideoPortCapsH

// ==========================================================================
// File:         TDDVideoPortCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortCaps Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"

// ==========================================================================
// Set:         TDDVideoPortCapsCaps_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortCaps::Caps property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Caps property description below. 
// ==========================================================================
enum TDDVideoPortCaps_Caps_Flags
    {
    ddvpcaps_autoflip,
    ddvpcaps_colorcontrol,
    ddvpcaps_interlaced,
    ddvpcaps_noninterlaced,
    ddvpcaps_oversampledvbi,
    ddvpcaps_readbackfield,
    ddvpcaps_readbackline,
    ddvpcaps_shareable,
    ddvpcaps_skipevenfields,
    ddvpcaps_skipoddfields,
    ddvpcaps_syncmaster,
    ddvpcaps_systemmemory,
    ddvpcaps_vbiandvideoindependent,
    ddvpcaps_vbisurface
    };
typedef Set< TDDVideoPortCaps_Caps_Flags, 
             ddvpcaps_autoflip, 
             ddvpcaps_vbisurface >  TDDVideoPortCaps_Caps_Set;
// ==========================================================================
// Set:         TDDVideoPortCapsFX_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortCaps::FX property using the BCB Object Inspector. 
//              For detailed descriptions of each flag, see the FX property 
//              description below. 
// ==========================================================================
enum TDDVideoPortCaps_FX_Flags
    {
    ddvpfx_croptopdata,
    ddvpfx_cropx,
    ddvpfx_cropy,
    ddvpfx_ignorevbixcrop,
    ddvpfx_interleave,
    ddvpfx_mirrorleftright,
    ddvpfx_mirrorupdown,
    ddvpfx_preshrinkx,
    ddvpfx_preshrinkxb,
    ddvpfx_preshrinkxs,
    ddvpfx_preshrinky,
    ddvpfx_preshrinkyb,
    ddvpfx_preshrinkys,
    ddvpfx_prestretchx,
    ddvpfx_prestretchxn,
    ddvpfx_prestretchy,
    ddvpfx_prestretchyn,
    ddvpfx_vbiconvert,
    ddvpfx_vbinointerleave,
    ddvpfx_vbinoscale
    };
typedef Set< TDDVideoPortCaps_FX_Flags, 
             ddvpfx_croptopdata, 
             ddvpfx_vbinoscale >  TDDVideoPortCaps_FX_Set;
// ==========================================================================
// Set:         TDDVideoPortCapsFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortCaps::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDVideoPortCaps_Flags_Flags
    {
    ddvpd_align,
    ddvpd_autoflip,
    ddvpd_caps,
    ddvpd_filterquality,
    ddvpd_fx,
    ddvpd_height,
    ddvpd_id,
    ddvpd_preferredautoflip,
    ddvpd_width
    };
typedef Set< TDDVideoPortCaps_Flags_Flags, 
             ddvpd_align, 
             ddvpd_width >  TDDVideoPortCaps_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortCaps
// Description: The TDDVideoPortCaps component wraps the DirectX 
//              DDVIDEOPORTCAPS structure, which contains the capabilities 
//              and alignment restrictions of a video port. 
//              
//              It is used by TDx_DrawVideoPortContainer::EnumVideoPorts() 
//              and the TDx_DrawVideoPortContainer::EnumVideoCallback event. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps_
    // Description: The Caps property defines flags indicating the available 
    //              video port capabilities. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortCaps_Caps_Set Caps_ = { read=FGetCaps_, 
    write=FSetCaps_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    FX_
    // Description: The FX property defines flags indicating the effects the 
    //              video port can support. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'FX' property via the BCB Object Inspector.  For 
    //              detailed descriptions of each flag, see the FX property 
    //              description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortCaps_FX_Set FX_ = { read=FGetFX_, write=FSetFX_, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating which 
    //              properties of this component contain valid data. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortCaps_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    AlignVideoPortBoundary
    // Description: The AlignVideoPortBoundary property defines where the 
    //              video port may be placed within a surface. 
    // ----------------------------------------------------------------------
    __property dword AlignVideoPortBoundary = { 
    read=FGetAlignVideoPortBoundary, write=FSetAlignVideoPortBoundary, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlignVideoPortCropBoundary
    // Description: The AlignVideoPortCropBoundary property defines a byte 
    //              multiplier used to determine legitimate positions for the 
    //              left cropping boundary of the video port. 
    // ----------------------------------------------------------------------
    __property dword AlignVideoPortCropBoundary = { 
    read=FGetAlignVideoPortCropBoundary, 
    write=FSetAlignVideoPortCropBoundary, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlignVideoPortCropWidth
    // Description: The AlignVideoPortCropWidth property defines the width 
    //              multiplier used to determine legitimate cropping widths 
    //              for the video port. 
    // ----------------------------------------------------------------------
    __property dword AlignVideoPortCropWidth = { 
    read=FGetAlignVideoPortCropWidth, write=FSetAlignVideoPortCropWidth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AlignVideoPortPrescaleWidth
    // Description: The AlignVideoPortPrescaleWidth property defines a width 
    //              multiplier used to determine legitimate widths for the 
    //              video port, after prescaling. 
    // ----------------------------------------------------------------------
    __property dword AlignVideoPortPrescaleWidth = { 
    read=FGetAlignVideoPortPrescaleWidth, 
    write=FSetAlignVideoPortPrescaleWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxHeight
    // Description: The MaxHeight property defines the maximum height allowed 
    //              for the video port field. 
    // ----------------------------------------------------------------------
    __property dword MaxHeight = { read=FGetMaxHeight, write=FSetMaxHeight, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxVBIWidth
    // Description: The MaxVBIWidth property defines the maximum width 
    //              allowed for the VBI data. 
    // ----------------------------------------------------------------------
    __property dword MaxVBIWidth = { read=FGetMaxVBIWidth, 
    write=FSetMaxVBIWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxWidth
    // Description: The MaxWidth property defines the maximum width allowed 
    //              for the video port field. 
    // ----------------------------------------------------------------------
    __property dword MaxWidth = { read=FGetMaxWidth, write=FSetMaxWidth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumAutoFlipSurfaces
    // Description: The NumAutoFlipSurfaces property defines the number of 
    //              autoflippable surfaces that are supported by the video 
    //              port. 
    //              When autoflipping surfaces are being used a flip will 
    //              occur every video port VSYNC unless the signal is being 
    //              interleaved in memory, in which case flip will occur 
    //              every second VSYNC. 
    // ----------------------------------------------------------------------
    __property dword NumAutoFlipSurfaces = { read=FGetNumAutoFlipSurfaces, 
    write=FSetNumAutoFlipSurfaces, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumFilterTapsX
    // Description: The NumFilterTapsX property defines the number of taps 
    //              used by the prescaler filter along the X axis. 
    //              0 indicates no prescaling, 1 indicates the prescaler 
    //              performs replication, 2 indicates the prescaler uses two 
    //              taps and so on. 
    // ----------------------------------------------------------------------
    __property word NumFilterTapsX = { read=FGetNumFilterTapsX, 
    write=FSetNumFilterTapsX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumFilterTapsY
    // Description: The NumFilterTapsY property defines the number of taps 
    //              used by the prescaler filter along the Y axis. 
    //              0 indicates no prescaling, 1 indicates the prescaler 
    //              performs replication, 2 indicates the prescaler uses two 
    //              taps and so on. 
    // ----------------------------------------------------------------------
    __property word NumFilterTapsY = { read=FGetNumFilterTapsY, 
    write=FSetNumFilterTapsY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumPreferredAutoflip
    // Description: The NumPreferredAutoFlip property defines the optimum 
    //              number of autoflippable surfaces supported. 
    //              When autoflipping is being used a flip will occur every 
    //              video port VSYNC unless the signal is being interleaved 
    //              in memory, in which case a flip will occur every second 
    //              VSYNC. 
    // ----------------------------------------------------------------------
    __property dword NumPreferredAutoflip = { read=FGetNumPreferredAutoflip, 
    write=FSetNumPreferredAutoflip, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumVBIAutoFlipSurfaces
    // Description: The NumVBIAutoFlipSurfaces property defines the number of 
    //              autoflippable surfaces capable of recieving VBI data that 
    //              are supported by the video port. 
    //              When autoflipping is being used a flip will occur every 
    //              video port VSYNC unless the signal is being interleaved 
    //              in memory, in which case a flip will occur every second 
    //              VSYNC. 
    //              The number of VBI surfaces in the flip chain must equal 
    //              the number of surfaces receiving the rest of the video 
    //              data. 
    // ----------------------------------------------------------------------
    __property dword NumVBIAutoFlipSurfaces = { 
    read=FGetNumVBIAutoFlipSurfaces, write=FSetNumVBIAutoFlipSurfaces, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PreshrinkXStep
    // Description: The PreshrinkXStep property defines the value used for 
    //              shrinking data along the X axis when TDDVideoPortCaps::FX 
    //              has the DDVPFX_PRESHRINKXS flag set. 
    // ----------------------------------------------------------------------
    __property dword PreshrinkXStep = { read=FGetPreshrinkXStep, 
    write=FSetPreshrinkXStep, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PreshrinkYStep
    // Description: The PreshrinkYStep property defines the value used for 
    //              shrinking data along the Y axis when TDDVideoPortCaps::FX 
    //              has the DDVPFX_PRESHRINKYS flag set. 
    // ----------------------------------------------------------------------
    __property dword PreshrinkYStep = { read=FGetPreshrinkYStep, 
    write=FSetPreshrinkYStep, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDVIDEOPORTCAPS) }; 

    // ----------------------------------------------------------------------
    // Property:    VideoPortID
    // Description: The VideoPortID property uniquely identifies the video 
    //              port to the system. 
    //              This property will contain an integer value between 0 and 
    //              TDDVideoPortCaps::MaxVideoPorts minus one. 
    // ----------------------------------------------------------------------
    __property dword VideoPortID = { read=FGetVideoPortID, 
    write=FSetVideoPortID, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps
    // Description: The Caps property defines flags indicating the available 
    //              video port capabilities. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPCAPS_AUTOFLIP -
    //                Support for flips that are performed automatically when 
    //                VREF's occur is available. 
    //                This is useful in avoiding tearing. Flips will occur on 
    //                every second VREF when the data is being interleaved in 
    //                memory. 
    //              DDVPCAPS_COLORCONTROL -
    //                Support for color control operations being performed on 
    //                incoming data before it is written to the frame buffer 
    //                is available. 
    //              DDVPCAPS_INTERLACED -
    //                Support for interlaced video is available. 
    //              DDVPCAPS_NONINTERLACED -
    //                Support for non interlaced video is available. 
    //              DDVPCAPS_OVERSAMPLEDVBI -
    //                Support for formats or widths of VBI data that are 
    //                different to those of the regular video data is 
    //                available. 
    //              DDVPCAPS_READBACKFIELD -
    //                Support for the use of 
    //                TDx_DrawVideoPort::GetFieldPolarity() is available. 
    //              DDVPCAPS_READBACKLINE -
    //                Support for the use of 
    //                TDx_DrawVideoPort::GetVideoLine() is available. 
    //              DDVPCAPS_SHAREABLE -
    //                Support for two genlocked video streams sharing the 
    //                video port with one stream using even fields and the 
    //                other using odd fields is available. 
    //                Separate properties such as address, scaling, cropping, 
    //                etc, are maintained for both fields. 
    //              DDVPCAPS_SKIPEVENFIELDS -
    //                Support for automatically discarding the even fields of 
    //                a video signal is available. 
    //              DDVPCAPS_SKIPODDFIELDS -
    //                Support for automatically discarding the odd fields of 
    //                a video signal is available. 
    //              DDVPCAPS_SYNCMASTER -
    //                Support for driving the display refresh rate with the 
    //                video port sync rate is available. 
    //              DDVPCAPS_SYSTEMMEMORY -
    //                Support for writing to system memory surfaces is 
    //                available. 
    //              DDVPCAPS_VBIANDVIDEOINDEPENDENT -
    //                Support for controlling the VBI and video portions of a 
    //                video stream with independent processes is available. 
    //              DDVPCAPS_VBISURFACE -
    //                Support for writing VBI data to a different surface 
    //                than the video data is available. 
    // ----------------------------------------------------------------------
    __property dword Caps = { read=FGetCaps, write=FSetCaps, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortCaps method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    FX
    // Description: The FX property defines flags indicating the effects the 
    //              video port can support. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPFX_CROPTOPDATA -
    //                Support for limited cropping of VBI data is available. 
    //              DDVPFX_CROPX -
    //                Support for cropping data along the X axis, before it 
    //                is written to the surface, is available. 
    //              DDVPFX_CROPY -
    //                Support for cropping data along the Y axis, before it 
    //                is written to the surface, is available. 
    //              DDVPFX_IGNOREVBIXCROP -
    //                Support for ignoring the X axis cropping coordinates 
    //                when cropping oversampled VBI data is available. 
    //              DDVPFX_INTERLEAVE -
    //                Support for interleaving the interlaced video fields in 
    //                memory is available. 
    //              DDVPFX_MIRRORLEFTRIGHT -
    //                Support for writing video data into the frame buffer, 
    //                while mirroring it from left to right, is available. 
    //              DDVPFX_MIRRORUPDOWN -
    //                Support for writing video data into the frame buffer, 
    //                while mirroring it from top to bottom, is available. 
    //              DDVPFX_PRESHRINKX -
    //                Support for shrinking data along the X axis by an 
    //                arbitary amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_PRESHRINKXB -
    //                Support for the binary shrinking of data along the X 
    //                axis, before writing it to the surface, is available. 
    //              DDVPFX_PRESHRINKXS -
    //                Support for shrinking data along the X axis by 
    //                increments of 1/XStep, before writing it to the 
    //                surface, is available. 
    //                The value for XStep is contained in 
    //                TDDVideoPortCaps::PreShrinkXStep. 
    //              DDVPFX_PRESHRINKY -
    //                Support for shrinking data along the Y axis by an 
    //                arbitary amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_PRESHRINKYB -
    //                Support for the binary shrinking of data along the Y 
    //                axis, before writing it to the surface, is available. 
    //              DDVPFX_PRESHRINKYS -
    //                Support for shrinking data along the Y axis by 
    //                increments of 1/YStep, before writing it to the 
    //                surface, is available. 
    //                The value for YStep is contained in 
    //                TDDVideoPortCaps::PreShrinkYStep. 
    //              DDVPFX_PRESTRETCHX -
    //                Support for stretching data along the X axis by an 
    //                arbitary amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_PRESTRETCHXN -
    //                Support for stretching data along the X axis by an 
    //                integer amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_PRESTRETCHY -
    //                Support for stretching data along the Y axis by an 
    //                arbitary amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_PRESTRETCHYN -
    //                Support for stretching data along the Y axis by an 
    //                integer amount, before writing it to the surface, is 
    //                available. 
    //              DDVPFX_VBICONVERT -
    //                Support for converting the VBI data independantly of 
    //                the remaining video data is available. 
    //              DDVPFX_VBINOINTERLEAVE -
    //                Support for disabling interleaving for the VBI data is 
    //                available. 
    //              DDVPFX_VBINOSCALE -
    //                Support for disabling scaling for the VBI data is 
    //                available. 
    // ----------------------------------------------------------------------
    __property dword FX = { read=FGetFX, write=FSetFX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating which 
    //              properties of this component contain valid data. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPD_ALIGN -
    //                TDDVideoPortCaps::AlignVideoPortBoundary, 
    //                TDDVideoPortCaps::AlignVideoPortPrescaleWidth, 
    //                TDDVideoPortCaps::AlignVideoPortCropBoundary and 
    //                TDDVideoPortCaps::AlignVideoPortCropWidth contain valid 
    //                data. 
    //              DDVPD_AUTOFLIP -
    //                TDDVideoPortCaps::NumAutoFlipSurfaces contains valid 
    //                data. 
    //              DDVPD_CAPS -
    //                TDDVideoPortCaps::Caps contains valid data. 
    //              DDVPD_FILTERQUALITY -
    //                TDDVideoPortCaps::NumFilterTapsX and 
    //                TDDVideoPortCaps::NumFilterTapsY contain valid data. 
    //              DDVPD_FX -
    //                TDDVideoPortCaps::FX contains valid data. 
    //              DDVPD_HEIGHT -
    //                TDDVideoPortCaps::MaxHeight contains valid data. 
    //              DDVPD_ID -
    //                TDDVideoPortCaps::VideoPortID contains valid data. 
    //              DDVPD_PREFERREDAUTOFLIP -
    //                TDDVideoPortCaps::NumPreferredAutoflip contains valid 
    //                data. 
    //              DDVPD_WIDTH -
    //                TDDVideoPortCaps::MaxWidth contains valid data. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortCaps's internal DDVIDEOPORTCAPS 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Caps_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps_Strings = { read=FGetCaps_Strings };

    // ----------------------------------------------------------------------
    // Property:    FX_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the FX property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* FX_Strings = { read=FGetFX_Strings };

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortCaps(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTCAPS* Internal_DDVIDEOPORTCAPS_Ptr = { 
    read=FGetInternal_DDVIDEOPORTCAPS_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTCAPS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDVideoPortCaps_Caps_Set __fastcall FGetCaps_(); 
    void __fastcall FSetCaps_( TDDVideoPortCaps_Caps_Set pCaps_ ); 
    TDDVideoPortCaps_FX_Set __fastcall FGetFX_(); 
    void __fastcall FSetFX_( TDDVideoPortCaps_FX_Set pFX_ ); 
    TDDVideoPortCaps_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDDVideoPortCaps_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetAlignVideoPortBoundary(); 
    void __fastcall FSetAlignVideoPortBoundary( dword pAlignVideoPortBoundary 
    ); 
    dword __fastcall FGetAlignVideoPortCropBoundary(); 
    void __fastcall FSetAlignVideoPortCropBoundary( dword 
    pAlignVideoPortCropBoundary ); 
    dword __fastcall FGetAlignVideoPortCropWidth(); 
    void __fastcall FSetAlignVideoPortCropWidth( dword 
    pAlignVideoPortCropWidth ); 
    dword __fastcall FGetAlignVideoPortPrescaleWidth(); 
    void __fastcall FSetAlignVideoPortPrescaleWidth( dword 
    pAlignVideoPortPrescaleWidth ); 
    dword __fastcall FGetCaps(); 
    void __fastcall FSetCaps( dword pCaps ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFX(); 
    void __fastcall FSetFX( dword pFX ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetMaxHeight(); 
    void __fastcall FSetMaxHeight( dword pMaxHeight ); 
    dword __fastcall FGetMaxVBIWidth(); 
    void __fastcall FSetMaxVBIWidth( dword pMaxVBIWidth ); 
    dword __fastcall FGetMaxWidth(); 
    void __fastcall FSetMaxWidth( dword pMaxWidth ); 
    dword __fastcall FGetNumAutoFlipSurfaces(); 
    void __fastcall FSetNumAutoFlipSurfaces( dword pNumAutoFlipSurfaces ); 
    word __fastcall FGetNumFilterTapsX(); 
    void __fastcall FSetNumFilterTapsX( word pNumFilterTapsX ); 
    word __fastcall FGetNumFilterTapsY(); 
    void __fastcall FSetNumFilterTapsY( word pNumFilterTapsY ); 
    dword __fastcall FGetNumPreferredAutoflip(); 
    void __fastcall FSetNumPreferredAutoflip( dword pNumPreferredAutoflip ); 
    dword __fastcall FGetNumVBIAutoFlipSurfaces(); 
    void __fastcall FSetNumVBIAutoFlipSurfaces( dword pNumVBIAutoFlipSurfaces 
    ); 
    dword __fastcall FGetPreshrinkXStep(); 
    void __fastcall FSetPreshrinkXStep( dword pPreshrinkXStep ); 
    dword __fastcall FGetPreshrinkYStep(); 
    void __fastcall FSetPreshrinkYStep( dword pPreshrinkYStep ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetVideoPortID(); 
    void __fastcall FSetVideoPortID( dword pVideoPortID ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetCaps_Strings();

    TStringList* __fastcall FGetFX_Strings();
    TStringList* __fastcall FGetFlags_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTCAPS* __fastcall FGetInternal_DDVIDEOPORTCAPS_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTCAPS( DDVIDEOPORTCAPS* 
    pDDVIDEOPORTCAPS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fCaps_Strings;
    TStringList* fFX_Strings;
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTCAPS fDDVIDEOPORTCAPS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
