#ifndef TDDVideoPortStatusH
#define TDDVideoPortStatusH

// ==========================================================================
// File:         TDDVideoPortStatus.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortStatus Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw3D_Library_Defns.H"
#include "TDx_Draw3D_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDDVideoPortStatus properties.
#include "TDDVideoPortConnect.H"

// ==========================================================================
// Set:         TDDVideoPortStatusFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortStatus::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDVideoPortStatus_Flags_Flags
    {
    ddvpstatus_vbionly,
    ddvpstatus_videoonly
    };
typedef Set< TDDVideoPortStatus_Flags_Flags, 
             ddvpstatus_vbionly, 
             ddvpstatus_videoonly >  TDDVideoPortStatus_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortStatus
// Description: The TDDVideoPortStatus component wraps the DirectX 
//              DDVIDEOPORTSTATUS structure, which describes the status of a 
//              video port. 
//              
//              It is used by 
//              TDx_DrawVideoPortContainer::QueryVideoPortStatus(). 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortStatus : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortStatus : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating which portion 
    //              of the video stream is being controlled by the video 
    //              port. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortStatus_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    InUse
    // Description: The InUse property will be set to true if the video port 
    //              is currently in use. 
    // ----------------------------------------------------------------------
    __property bool InUse = { read=FGetInUse, write=FSetInUse, default=false 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of this 
    //              component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDVIDEOPORTSTATUS) }; 

    // ----------------------------------------------------------------------
    // Property:    VideoPortType
    // Description: The VideoPortType property references a 
    //              TDDVideoPortConnect component describing the video port 
    //              connection information. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortConnect* VideoPortType = { read=FGetVideoPortType, 
    write=FSetVideoPortType, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortStatus component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortStatus method or 
    //              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=D3D_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating which portion 
    //              of the video stream is being controlled by the video 
    //              port. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPSTATUS_VBIONLY -
    //                The VBI portion of the video stream is being controlled 
    //                by the video port. 
    //              DDVPSTATUS_VIDEOONLY -
    //                The normal portion of the video stream is being 
    //                controlled by the video port. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortStatus's internal DDVIDEOPORTSTATUS 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortStatus(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortStatus(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTSTATUS* Internal_DDVIDEOPORTSTATUS_Ptr = { 
    read=FGetInternal_DDVIDEOPORTSTATUS_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTSTATUS_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDVideoPortStatus_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDDVideoPortStatus_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    bool __fastcall FGetInUse(); 
    void __fastcall FSetInUse( bool pInUse ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    TDDVideoPortConnect* __fastcall FGetVideoPortType(); 
    void __fastcall FSetVideoPortType( TDDVideoPortConnect* pVideoPortType ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTSTATUS* __fastcall FGetInternal_DDVIDEOPORTSTATUS_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTSTATUS( DDVIDEOPORTSTATUS* 
    pDDVIDEOPORTSTATUS ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDDVideoPortConnect* fVideoPortType; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTSTATUS fDDVIDEOPORTSTATUS; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
