// --------------------------------------------------------------------------

// ==========================================================================
// File:         TDx_DrawClipper.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file contains the code for the TDx_DrawClipper 
//               Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TDx_DrawClipper.H"
// --------------------------------------------------------------------------
// external classes used by TDx_DrawClipper methods.
#include "TDx_Draw.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_Draw3D_Library_Defns"
#pragma link "TDx_Draw3D_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TDx_DrawClipper*)
{
    new TDx_DrawClipper(NULL);
}
// --------------------------------------------------------------------------
namespace Tdx_drawclipper
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TDx_DrawClipper)};
        RegisterComponents("TDx_Draw3D", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Constructor: TDx_DrawClipper::TDx_DrawClipper()
// Description: The default constructor for the TDx_DrawClipper object.
// --------------------------------------------------------------------------
__fastcall TDx_DrawClipper::TDx_DrawClipper(TComponent* Owner)
    : TComponent(Owner)
{
fCreated = false;
fErrorValue = D3D_OK;
}
// --------------------------------------------------------------------------
// Destructor:  TDx_DrawClipper::~TDx_DrawClipper()
// Description: The destructor for the TDx_DrawClipper object.
// --------------------------------------------------------------------------
__fastcall TDx_DrawClipper::~TDx_DrawClipper()
{
// destroy internals
if (Created) Destroy();
}
// --------------------------------------------------------------------------
// Property:    Created
// Description: The Created property is true if the internal 
//              LPDIRECTDRAWCLIPPER used in this component has been 
//              successfully created, otherwise Created is false. 
//              
//              To create the internal LPDIRECTDRAWCLIPPER, call the 
//              TDx_DrawClipper::Create() method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::FGetCreated()
{
return fCreated;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawClipper::FSetCreated( bool pCreated )
{
fCreated = (pCreated && (fLPDIRECTDRAWCLIPPER==NULL));
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TDx_DrawClipper method or 
//              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_DrawClipper::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawClipper::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Method:      Create()
// Description: The Create() method is used to automatically create the 
//              internal LPDIRECTDRAWCLIPPER interface used in the 
//              TDx_DrawClipper component and must be called before any 
//              methods of the TDx_DrawClipper component will function. 
// Params:      pDraw - 
//                A TDx_Draw object that will manage the clipper. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::Create( TDx_Draw* pDraw )
{
// if the component internals have already been created, exit
if (fCreated)
    {
    fErrorValue = TDX_ALREADYCREATED;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ALREADYCREATED", "The "+Name+"::Create() method was called when the component has already been created successfully." );
    return false;
    }

// check params for accuracy
if (pDraw==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_BADPARAMS", "The 'pDraw' parameter must point to an existing TDx_Draw component and cannot be NULL." );
    return false;
    }

// code that creates the internal clipper
pDraw->CreateClipper( this );
fErrorValue = pDraw->ErrorValue;

// check for error result
if (fErrorValue!=DD_OK)
    {
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ERROR", Name+"::Create() failed with Error: "+TDx_Draw_Library_ErrorString(fErrorValue)+", "+TDx_Draw_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// successfully created
fCreated = true;
if (FOnCreate) FOnCreate(this);
return true;
}

// --------------------------------------------------------------------------
// Method:      Destroy()
// Description: The Destroy() method is used to automatically destroy the 
//              internal LPDIRECTDRAWCLIPPER interface used in the 
//              TDx_DrawClipper component and should be called when the 
//              internal interface is no longer required. 
//              
//              Note: This method is called by the component's destructor. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::Destroy()
{
// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_NOTCREATED", "The "+Name+"::Destroy() method was called before being created successfully." );
    return false;
    }

// code that destroys the internal interfaces
if (FOnDestroy) FOnDestroy(this);

if (fLPDIRECTDRAWCLIPPER==NULL) 
    { 
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "The "+Name+" component's internal LPDIRECTDRAWCLIPPER is NULL. Aborting the Destroy()." );
    return false;
    }

try { 
    fLPDIRECTDRAWCLIPPER->Release(); 
    } 
catch (...) 
    { 
    fErrorValue = TDX_ERROR; 
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "An exception occurred in LPDIRECTDRAWCLIPPER->Release(). Check you are destroying all components in reverse order of creation." );
    }

// successful destruction
fLPDIRECTDRAWCLIPPER = NULL;
fCreated = false;
return true;
}

// --------------------------------------------------------------------------
// Method:      GetClipList()
// Description: The TDx_DrawClipper::GetClipList method will retrieve a copy 
//              of this clipper's clip list. 
//              
//              Subsets of a clip list can be retrieved using the Rect 
//              parameter of this method to pass a rectangle which clips the 
//              clip list. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_GENERIC 
//              DDERR_INVALIDCLIPLIST 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_NOCLIPLIST 
//              DDERR_REGIONTOOSMALL 
// Params:      pRect - 
//                The Rect parameter references a TRect for clipping the clip 
//                list. 
//                Set this parameter to NULL to retrieve the whole clip list. 
//              pClipList - 
//                The ClipList parameter references a RGNDATA structure for 
//                holding the clip list if this method returns successfully. 
//                When this parameter is set to NULL the Size parameter of this 
//                method will be filled with the size, in bytes, of the entire 
//                clip list. 
//              pSize - 
//                The Size parameter defines the size, in bytes, of the buffer 
//                referenced by the ClipList parameter of this method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::GetClipList( TRect* pRect, RGNDATA* pClipList, dword* pSize )
{
// Original Function Definition
// HRESULT GetClipList( 
//   LPRECT lpRect,         
//   LPRGNDATA lpClipList,   
//   LPDWORD lpdwSize       
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetClipList()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pSize==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetClipList()", "TDX_BADPARAMS", "'pSize' cannot be 'NULL'");
    return false;
    }

// Translate Parameters before calling Original Function
RECT TempRect;
LPRECT TempRect_Ptr;
if (pRect==NULL)
    {
    TempRect_Ptr = NULL;
    }
 else
    {
    TempRect.top = pRect->Top;
    TempRect.left = pRect->Left;
    TempRect.right = pRect->Right;
    TempRect.bottom = pRect->Bottom;
    TempRect_Ptr = &TempRect;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWCLIPPER->GetClipList( TempRect_Ptr, (LPRGNDATA) pClipList, (LPDWORD) pSize );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetClipList()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetHWnd()
// Description: The TDx_DrawClipper::GetHWnd method will return the window 
//              handle associated with this clipper by 
//              TDx_DrawClipper::SetHWnd(). 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pWnd - 
//                The Wnd parameter will reference the clipper's previously 
//                associated window handle if this method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::GetHWnd( HWND* pWnd )
{
// Original Function Definition
// HRESULT GetHWnd( 
//   HWND FAR *lphWnd   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetHWnd()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pWnd==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetHWnd()", "TDX_BADPARAMS", "'pWnd' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWCLIPPER->GetHWnd( pWnd );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetHWnd()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      IsClipListChanged()
// Description: The TDx_DrawClipper::IsClipListChanged method will indicate 
//              whether the clip list has been changed. 
//              
//              This method is only useful when the clipper is associated 
//              with a window handle. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pChanged - 
//                The Changed parameter will reference the current state of the 
//                clip list if this method returns successfully. 
//                TRUE if the clip list has changed. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::IsClipListChanged( bool* pChanged )
{
// Original Function Definition
// HRESULT IsClipListChanged( 
//   BOOL FAR *lpbChanged   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::IsClipListChanged()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pChanged==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::IsClipListChanged()", "TDX_BADPARAMS", "'pChanged' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWCLIPPER->IsClipListChanged( (BOOL*) pChanged );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::IsClipListChanged()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      SetClipList()
// Description: The TDx_DrawClipper::SetClipList method will set or delete 
//              the clip lists used by TDx_DrawSurface::Blt() and 
//              TDx_DrawSurface::UpdateOverlay for surfaces to which the 
//              clipper is attached. 
//              
//              A clip list cannot be set when there is a window handle 
//              associated with the clipper. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_CLIPPERISUSINGHWND 
//              DDERR_INVALIDCLIPLIST 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_OUTOFMEMORY 
// Params:      pClipList - 
//                The ClipList parameter references a RGNDATA structure holding 
//                the clip list that is to be set. 
//                Setting this property to NULL when a clip list already exists 
//                will delete that clip list. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::SetClipList( RGNDATA* pClipList )
{
// Original Function Definition
// HRESULT SetClipList( 
//   LPRGNDATA lpClipList,   
//   DWORD dwFlags           
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::SetClipList()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWCLIPPER->SetClipList( (LPRGNDATA) pClipList, 0 );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::SetClipList()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      SetHWnd()
// Description: The TDx_DrawClipper::SetHWnd method will set the window 
//              handle used to obtain the clipping information. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDCLIPLIST 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_OUTOFMEMORY 
// Params:      pWnd - 
//                The Wnd parameter defines the window handle to be used for 
//                obtaining clip information. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawClipper::SetHWnd( HWND pWnd )
{
// Original Function Definition
// HRESULT SetHWnd( 
//   DWORD dwFlags,   
//   HWND hWnd       
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::SetHWnd()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pWnd==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::SetHWnd()", "TDX_BADPARAMS", "'pWnd' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWCLIPPER->SetHWnd( 0, pWnd );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::SetHWnd()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Internal Interface Access
// --------------------------------------------------------------------------
LPDIRECTDRAWCLIPPER __fastcall TDx_DrawClipper::FGetInternal_LPDIRECTDRAWCLIPPER()
{
return fLPDIRECTDRAWCLIPPER;
}
// --------------------------------------------------------------------------
LPDIRECTDRAWCLIPPER* __fastcall TDx_DrawClipper::FGetInternal_LPDIRECTDRAWCLIPPER_Ptr()
{
return &fLPDIRECTDRAWCLIPPER;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawClipper::FSetInternal_LPDIRECTDRAWCLIPPER( LPDIRECTDRAWCLIPPER pLPDIRECTDRAWCLIPPER )
{
if (!fCreated)
    {
    fLPDIRECTDRAWCLIPPER = pLPDIRECTDRAWCLIPPER;
    fCreated = (fLPDIRECTDRAWCLIPPER!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawClipper::Internal_LPDIRECTDRAWCLIPPER_Update()
{
fCreated = (fLPDIRECTDRAWCLIPPER!=NULL);
}

// --------------------------------------------------------------------------
