// --------------------------------------------------------------------------

// ==========================================================================
// File:         TDx_DrawVideoPort.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file contains the code for the TDx_DrawVideoPort 
//               Component 
//
// "TDx_Draw3D_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TDx_DrawVideoPort.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawVideoPort component.
#include "TDx_DrawVideoPortContainer.H"
#include "TDDVideoPortDesc.H"
// --------------------------------------------------------------------------
// external classes used by TDx_DrawVideoPort methods.
#include "TDx_DrawSurface.H"
#include "TDDPixelFormat.H"
#include "TDDVideoPortBandwidth.H"
#include "TDDColorControl.H"
#include "TDDVideoPortInfo.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_Draw3D_Library_Defns"
#pragma link "TDx_Draw3D_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TDx_DrawVideoPort*)
{
    new TDx_DrawVideoPort(NULL);
}
// --------------------------------------------------------------------------
namespace Tdx_drawvideoport
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TDx_DrawVideoPort)};
        RegisterComponents("TDx_Draw3D", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Constructor: TDx_DrawVideoPort::TDx_DrawVideoPort()
// Description: The default constructor for the TDx_DrawVideoPort object.
// --------------------------------------------------------------------------
__fastcall TDx_DrawVideoPort::TDx_DrawVideoPort(TComponent* Owner)
    : TComponent(Owner)
{
fCreated = false;
fErrorValue = D3D_OK;
}
// --------------------------------------------------------------------------
// Destructor:  TDx_DrawVideoPort::~TDx_DrawVideoPort()
// Description: The destructor for the TDx_DrawVideoPort object.
// --------------------------------------------------------------------------
__fastcall TDx_DrawVideoPort::~TDx_DrawVideoPort()
{
// destroy internals
if (Created) Destroy();
}
// --------------------------------------------------------------------------
// Property:    Created
// Description: The Created property is true if the internal 
//              LPDIRECTDRAWVIDEOPORT used in this component has been 
//              successfully created, otherwise Created is false. 
//              
//              To create the internal LPDIRECTDRAWVIDEOPORT, call the 
//              TDx_DrawVideoPort::Create() method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::FGetCreated()
{
return fCreated;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawVideoPort::FSetCreated( bool pCreated )
{
fCreated = (pCreated && (fLPDIRECTDRAWVIDEOPORT==NULL));
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TDx_DrawVideoPort method or 
//              fget/fset. eg. D3D_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_DrawVideoPort::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawVideoPort::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Method:      Create()
// Description: The Create() method is used to automatically create the 
//              internal LPDIRECTDRAWVIDEOPORT interface used in the 
//              TDx_DrawVideoPort component and must be called before any 
//              methods of the TDx_DrawVideoPort component will function. 
// Params:      pFlags - 
//                Flags specifying video-port control options. This parameter 
//                can be NULL if control options are not needed: 
//                Flags:
//                  DDVPCREATE_VBIONLY -
//                    The process wants to obtain control of the VBI period 
//                    of the video stream. 
//                  DDVPCREATE_VIDEOONLY -
//                    The process wants to obtain control of the video period 
//                    of the video stream. 
//              pVideoPortDesc - 
//                The VideoPortDesc parameter describes the videoport to be 
//                created. 
//              pVideoPortContainer - 
//                The VideoPortContainer that will manage the created 
//                videoport. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::Create( dword pFlags, TDDVideoPortDesc* pVideoPortDesc, TDx_DrawVideoPortContainer* pVideoPortContainer )
{
// if the component internals have already been created, exit
if (fCreated)
    {
    fErrorValue = TDX_ALREADYCREATED;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ALREADYCREATED", "The "+Name+"::Create() method was called when the component has already been created successfully." );
    return false;
    }

// check params for accuracy
if (pVideoPortDesc==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_BADPARAMS", "The 'pVideoPortDesc' parameter must point to an existing TDDVideoPortDesc component and cannot be NULL." );
    return false;
    }
if (pVideoPortContainer==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_BADPARAMS", "The 'pVideoPortContainer' parameter must point to an existing TDx_DrawVideoPortContainer component and cannot be NULL." );
    return false;
    }

// code that creates the internal interface
pVideoPortContainer->CreateVideoPort( pFlags, pVideoPortDesc, this );
fErrorValue = pVideoPortContainer->ErrorValue;

// check for error result
if (fErrorValue!=DD_OK)
    {
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ERROR", Name+"::Create() failed with Error: "+TDx_Draw_Library_ErrorString(fErrorValue)+", "+TDx_Draw_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// successfully created
fCreated = true;
if (FOnCreate) FOnCreate(this);
return true;
}

// --------------------------------------------------------------------------
// Method:      Destroy()
// Description: The Destroy() method is used to automatically destroy the 
//              internal LPDIRECTDRAWVIDEOPORT interface used in the 
//              TDx_DrawVideoPort component and should be called when the 
//              internal interface is no longer required. 
//              
//              Note: This method is called by the component's destructor. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::Destroy()
{
// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_NOTCREATED", "The "+Name+"::Destroy() method was called before being created successfully." );
    return false;
    }

// code that destroys the internal interfaces
if (FOnDestroy) FOnDestroy(this);

if (fLPDIRECTDRAWVIDEOPORT==NULL) 
    { 
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "The "+Name+" component's internal LPDIRECTDRAWVIDEOPORT is NULL. Aborting the Destroy()." );
    return false;
    }

try { 
    fLPDIRECTDRAWVIDEOPORT->Release(); 
    } 
catch (...) 
    { 
    fErrorValue = TDX_ERROR; 
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "An exception occurred in LPDIRECTDRAWVIDEOPORT->Release(). Check you are destroying all components in reverse order of creation." );
    }

// successful destruction
fLPDIRECTDRAWVIDEOPORT = NULL;
fCreated = false;
return true;
}

// --------------------------------------------------------------------------
// Method:      Flip()
// Description: The TDx_DrawVideoPort::Flip method will cause the video port 
//              to use a new surface for the next frame of video data. 
//              
//              This can be useful for preventing artifacts such as tearing. 
//              
//              Calls to TDx_DrawVideoPort::Flip() are asynchronous but the 
//              flips generated are synchronized with the video signals VBI. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pSurface - 
//                The Surface parameter references a surface to receive the 
//                next video frame. 
//                Set this parameter to NULL to cycle through the flip chain in 
//                the normal attachment order. 
//              pFlags - 
//                The Flags parameter defines flags indicating the flip 
//                options. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPFLIP_VBI -
//                    The surface referenced by the Surface property of this 
//                    method will receive VBI video data. 
//                  DDVPFLIP_VIDEO -
//                    The surface referenced by the Surface property of this 
//                    method will receive normal video data. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::Flip( TDx_DrawSurface* pSurface, dword pFlags )
{
// Original Function Definition
// HRESULT Flip( 
//   LPDIRECTDRAWSURFACE lpDDSurface,   
//   DWORD dwFlags                     
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Flip()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->Flip( (pSurface==NULL) ? NULL : pSurface->Internal_LPDIRECTDRAWSURFACE, (DWORD) pFlags );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::Flip()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetBandwidthInfo()
// Description: The TDx_DrawVideoPort::GetBandwidthInfo method will retrieve 
//              the overlay zoom factors and device limitations for a video 
//              port outputting the specified pixel format. 
//              
//              This method is usually called twice. Once with DDVPB_TYPE set 
//              to determine whether the bandwidth information is provided in 
//              terms of the destination or source overlay size and again 
//              with either of the other flags set as indicated by the 
//              previous call to retrieve the required information. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pFormat - 
//                The Format parameter references a TDDPixelFormat component 
//                defining the output pixel format for which bandwidth 
//                information is being requested. 
//              pWidth - 
//                The Width parameter defines the horizontal dimension of an 
//                overlay or the video data width. 
//                Interpretation of this value depends on which flags are set 
//                in the Flags parameter of this method. 
//              pHeight - 
//                The Height parameter defines the vertical dimension of an 
//                overlay or the video data height. 
//                Interpretation of this value depends on which flags are set 
//                in the Flags parameter of this method. 
//              pFlags - 
//                The Flags parameter defines flags indicating how the Width 
//                and Height parameters of this method are to be interpreted. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPB_OVERLAY -
//                    The Width and Height parameters of this method define 
//                    the source overlay surface's size. 
//                    This flag should be set when the video port is 
//                    dependent on the source overlay size. 
//                  DDVPB_TYPE -
//                    Retrieve the device's dependency type in the 
//                    TDDVideoPortBandwidth::Caps referenced by the Bandwidth 
//                    parameter of this method. 
//                    This flag should be used on the first call to this 
//                    method. 
//                    The Width and Height parameters of this method will be 
//                    empty. 
//                  DDVPB_VIDEOPORT -
//                    The Width and Height parameters define the prescale 
//                    size of the video data written to the frame buffer by 
//                    the video port. 
//                    This flag should be set when the video port is 
//                    dependent on the overlay zoom factor. 
//              pBandwidth - 
//                The Bandwidth parameter references a TDDVideoPortBandwidth 
//                component for holding the retrieved bandwidth and device 
//                dependency information if this method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::GetBandwidthInfo( TDDPixelFormat* pFormat, dword pWidth, dword pHeight, dword pFlags, TDDVideoPortBandwidth* pBandwidth )
{
// Original Function Definition
// HRESULT GetBandwidthInfo( 
//   LPDDPIXELFORMAT lpddpfFormat,       
//   DWORD dwWidth,                       
//   DWORD dwHeight,                     
//   DWORD dwFlags,                       
//   LPDDVIDEOPORTBANDWIDTH lpBandwidth   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetBandwidthInfo()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pFormat==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetBandwidthInfo()", "TDX_BADPARAMS", "'pFormat' cannot be 'NULL'");
    return false;
    }
if (pBandwidth==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetBandwidthInfo()", "TDX_BADPARAMS", "'pBandwidth' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetBandwidthInfo( pFormat->Internal_DDPIXELFORMAT_Ptr, (DWORD) pWidth, (DWORD) pHeight, (DWORD) pFlags, pBandwidth->Internal_DDVIDEOPORTBANDWIDTH_Ptr );

// Translate Data returned from Function
pFormat->Internal_DDPIXELFORMAT_Update();
pBandwidth->Internal_DDVIDEOPORTBANDWIDTH_Update();

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetBandwidthInfo()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetColorControls()
// Description: The TDx_DrawVideoPort::GetColorControls method will retrieve 
//              the video port's current color control settings. 
//              
//              The color control options supported by the video port can be 
//              determined from TDDColorControl::Flags. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
// Params:      pColorControl - 
//                The ColorControl parameter references a TDDColorControl 
//                component for holding the video port's current color control 
//                settings if this method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::GetColorControls( TDDColorControl* pColorControl )
{
// Original Function Definition
// HRESULT GetColorControls( 
//   LPDDCOLORCONTROL lpColorControl   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetColorControls()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pColorControl==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetColorControls()", "TDX_BADPARAMS", "'pColorControl' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetColorControls( pColorControl->Internal_DDCOLORCONTROL_Ptr );

// Translate Data returned from Function
pColorControl->Internal_DDCOLORCONTROL_Update();

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetColorControls()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetFieldPolarity()
// Description: The TDx_DrawVideoPort::GetFieldPolarity method will indicate 
//              if the current field of interlaced video is odd or even. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
//              DDERR_VIDEONOTACTIVE 
// Params:      pFieldPolarity - 
//                The FieldPolarity parameter references a boolean that will be 
//                set to TRUE if the current field is even and FALSE if the 
//                current field is odd. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::GetFieldPolarity( bool* pFieldPolarity )
{
// Original Function Definition
// HRESULT GetFieldPolarity( 
//   LPBOOL lpbFieldPolarity   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetFieldPolarity()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pFieldPolarity==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetFieldPolarity()", "TDX_BADPARAMS", "'pFieldPolarity' cannot be 'NULL'");
    return false;
    }

// Translate Parameters before calling Original Function
BOOL TempFieldPolarity;
LPBOOL TempFieldPolarity_Ptr;
(pFieldPolarity) ? TempFieldPolarity=TRUE : TempFieldPolarity=FALSE;
TempFieldPolarity_Ptr = &TempFieldPolarity;

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetFieldPolarity( TempFieldPolarity_Ptr );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetFieldPolarity()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetInputFormats()
// Description: The TDx_DrawVideoPort::GetInputFormats method will retrieve 
//              the supported video port input pixel formats. 
//              
//              Setting the Formats parameter to NULL will cause the number 
//              of supported input formats to be returned in the NumFormats 
//              parameter. 
//              
//              If this method call fails, the one of the following values 
//              will be returned: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_MOREDATA 
// Params:      pNumFormats - 
//                The NumFormats parameter references the size of the array 
//                referenced by the Formats parameter of this method. 
//                When the size needed exceeds this value, the array will be 
//                filled, this parameter set to the actual size needed and a 
//                DDERR_MOREDATA error returned. 
//              pFormats - 
//                The Formats parameter references an array of TDDPixelFormat 
//                components for holding the retrieved input pixel formats if 
//                this method returns successfully. 
//                Setting this parameter to NULL will cause the required array 
//                size to be returned in the NumFormats parameter of this 
//                method. 
//              pFlags - 
//                The Flags parameter defines flags indicating which portion of 
//                the video signal to enumerate formats for. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPFORMAT_VBI -
//                    Retrieve the format of the VBI data. 
//                  DDVPFORMAT_VIDEO -
//                    Retrieve the format of the normal video data. 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_DrawVideoPort::GetInputFormats( dword* pNumFormats, TDDPixelFormat* pFormats, dword pFlags )
{
// Original Function Definition
// HRESULT GetInputFormats( 
//   LPDWORD lpNumFormats,       
//   LPDDPIXELFORMAT lpFormats,   
//   DWORD dwFlags               
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetInputFormats()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pNumFormats==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetInputFormats()", "TDX_BADPARAMS", "'pNumFormats' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetInputFormats( (LPDWORD) pNumFormats, (pFormats==NULL) ? NULL : pFormats->Internal_DDPIXELFORMAT_Ptr, (DWORD) pFlags );

// Translate Data returned from Function
if (pFormats!=NULL) pFormats->Internal_DDPIXELFORMAT_Update();

// Success!
return fErrorValue;
}


// --------------------------------------------------------------------------
// Method:      GetOutputFormats()
// Description: The TDx_DrawVideoPort::GetOutputFormats method will retrieve 
//              the supported video port output pixel formats for a specified 
//              input format. 
//              
//              If this method call fails, the one of the following values 
//              will be returned: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_MOREDATA 
// Params:      pInputFormat - 
//                The InputFormat parameter references a TDDPixelFormat 
//                component holding the input format for which conversion 
//                information is desired. 
//              pNumFormats - 
//                The NumFormats parameter references the size of the array 
//                referenced by the Formats parameter of this method. 
//                When the size needed exceeds this value, the array will be 
//                filled, this parameter set to the actual size needed and a 
//                DDERR_MOREDATA error returned. 
//              pFormats - 
//                The Formats parameter references an array of TDDPixelFormat 
//                components for holding the retrieved output pixel formats if 
//                this method returns successfully. 
//                Setting this parameter to NULL will cause the required array 
//                size to be returned in the NumFormats parameter of this 
//                method. 
//              pFlags - 
//                The Flags parameter defines flags indicating the portion of 
//                the video signal for which the format is to be enumerated. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPFORMAT_VBI -
//                    Retrieve the format of the VBI data. 
//                  DDVPFORMAT_VIDEO -
//                    Retrieve the format of the normal video data. 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_DrawVideoPort::GetOutputFormats( TDDPixelFormat* pInputFormat, dword* pNumFormats, TDDPixelFormat* pFormats, dword pFlags )
{
// Original Function Definition
// HRESULT GetOutputFormats( 
//   LPDDPIXELFORMAT lpInputFormat,   
//   LPDWORD lpNumFormats,       
//   LPDDPIXELFORMAT lpFormats,   
//   DWORD dwFlags               
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetOutputFormats()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pInputFormat==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetOutputFormats()", "TDX_BADPARAMS", "'pInputFormat' cannot be 'NULL'");
    return false;
    }
if (pNumFormats==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetOutputFormats()", "TDX_BADPARAMS", "'pNumFormats' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetOutputFormats( pInputFormat->Internal_DDPIXELFORMAT_Ptr, (LPDWORD) pNumFormats, (pFormats==NULL) ? NULL : pFormats->Internal_DDPIXELFORMAT_Ptr, (DWORD) pFlags );

// Translate Data returned from Function
pInputFormat->Internal_DDPIXELFORMAT_Update();
if (pFormats!=NULL) pFormats->Internal_DDPIXELFORMAT_Update();

// Success!
return fErrorValue;
}


// --------------------------------------------------------------------------
// Method:      GetVideoLine()
// Description: The TDx_DrawVideoPort::GetVideoLine method will retrieve the 
//              scan line number currently being written to the frame buffer. 
//              
//              This number is the true video line, relative to the field 
//              height, before prescaling. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
//              DDERR_VERTICALBLANKINPROGRESS 
//              DDERR_VIDEONOTACTIVE 
// Params:      pLine - 
//                The Line parameter will reference the current scan line 
//                number if this method returns successfully. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::GetVideoLine( dword* pLine )
{
// Original Function Definition
// HRESULT GetVideoLine( 
//   LPDWORD lpdwLine   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetVideoLine()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pLine==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetVideoLine()", "TDX_BADPARAMS", "'pLine' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetVideoLine( (LPDWORD) pLine );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetVideoLine()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetVideoSignalStatus()
// Description: The TDx_DrawVideoPort::GetVideoSignalStatus method will 
//              determine the current video signal status. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
// Params:      pStatus - 
//                The Status parameter references a variable defining flags 
//                indicating the current video signal status. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPSQ_NOSIGNAL -
//                    There is no video signal coming to the video port. 
//                  DDVPSQ_SIGNALOK -
//                    There is a valid video signal coming to the video port. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::GetVideoSignalStatus( dword* pStatus )
{
// Original Function Definition
// HRESULT GetVideoSignalStatus( 
//   LPDWORD lpdwStatus   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetVideoSignalStatus()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pStatus==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::GetVideoSignalStatus()", "TDX_BADPARAMS", "'pStatus' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->GetVideoSignalStatus( (LPDWORD) pStatus );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetVideoSignalStatus()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      SetColorControls()
// Description: The TDx_DrawVideoPort::SetColorControls method will set the 
//              video port color controls. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
// Params:      pColorControl - 
//                The ColorControl parameter references a TDDColorControl 
//                component holding the new video port color control settings. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::SetColorControls( TDDColorControl* pColorControl )
{
// Original Function Definition
// HRESULT SetColorControls( 
//   LPDDCOLORCONTROL lpColorControl   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::SetColorControls()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pColorControl==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::SetColorControls()", "TDX_BADPARAMS", "'pColorControl' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->SetColorControls( pColorControl->Internal_DDCOLORCONTROL_Ptr );

// Translate Data returned from Function
pColorControl->Internal_DDCOLORCONTROL_Update();

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::SetColorControls()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      SetTargetSurface()
// Description: The TDx_DrawVideoPort::SetTargetSurface method will set the 
//              target surface for receiving video and/or VBI data. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pSurface - 
//                The Surface parameter references the surface which will 
//                receive video and/or VBI data. 
//              pFlags - 
//                The Flags parameter defines flags indicating the type of 
//                target surface. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPTARGET_VBI -
//                    The surface will receive VBI data only. 
//                  DDVPTARGET_VIDEO -
//                    The surface will receive both video and VBI data unless 
//                    a separate surface for VBI data has been attached. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::SetTargetSurface( TDx_DrawSurface* pSurface, dword pFlags )
{
// Original Function Definition
// HRESULT SetTargetSurface( 
//   LPDIRECTDRAWSURFACE lpDDSurface,   
//   DWORD dwFlags                     
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::SetTargetSurface()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pSurface==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::SetTargetSurface()", "TDX_BADPARAMS", "'pSurface' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->SetTargetSurface( pSurface->Internal_LPDIRECTDRAWSURFACE, (DWORD) pFlags );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::SetTargetSurface()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      StartVideo()
// Description: The TDx_DrawVideoPort::StartVideo method will start the flow 
//              of live video data from the video port hardware into the 
//              target surface. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_SURFACELOST 
// Params:      pVideoInfo - 
//                The VideoInfo parameter references a TDDVideoPortInfo 
//                component defining the video transfer properties. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::StartVideo( TDDVideoPortInfo* pVideoInfo )
{
// Original Function Definition
// HRESULT StartVideo( 
//   LPDDVIDEOPORTINFO lpVideoInfo   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::StartVideo()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pVideoInfo==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::StartVideo()", "TDX_BADPARAMS", "'pVideoInfo' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->StartVideo( pVideoInfo->Internal_DDVIDEOPORTINFO_Ptr );

// Translate Data returned from Function
pVideoInfo->Internal_DDVIDEOPORTINFO_Update();

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::StartVideo()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      StopVideo()
// Description: The TDx_DrawVideoPort::StopVideo method will stop the sending 
//              of video data to the frame buffer. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with the following value: 
//              DDERR_INVALIDOBJECT 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::StopVideo()
{
// Original Function Definition
// HRESULT StopVideo(); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::StopVideo()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->StopVideo(  );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::StopVideo()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      UpdateVideo()
// Description: The TDx_DrawVideoPort::UpdateVideo method will modify the 
//              transfer properties governing the flow of video data from the 
//              hardware video port to a surface. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
// Params:      pVideoInfo - 
//                The VideoInfo parameter references a TDDVideoPortInfo 
//                component defining the video transfer properties. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::UpdateVideo( TDDVideoPortInfo* pVideoInfo )
{
// Original Function Definition
// HRESULT UpdateVideo( 
//   LPDDVIDEOPORTINFO lpVideoInfo   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::UpdateVideo()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pVideoInfo==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::UpdateVideo()", "TDX_BADPARAMS", "'pVideoInfo' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->UpdateVideo( pVideoInfo->Internal_DDVIDEOPORTINFO_Ptr );

// Translate Data returned from Function
pVideoInfo->Internal_DDVIDEOPORTINFO_Update();

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::UpdateVideo()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      WaitForSync()
// Description: The TDx_DrawVideoPort::WaitForSync method can help an 
//              application synchronise by waiting for the VSYNC or a 
//              specified scan line. 
//              
//              If the method call fails, the OnError event will be triggered 
//              with one of the following values: 
//              DDERR_INVALIDOBJECT 
//              DDERR_INVALIDPARAMS 
//              DDERR_UNSUPPORTED 
//              DDERR_VIDEONOTACTIVE 
//              DDERR_WASSTILLDRAWING 
// Params:      pFlags - 
//                The Flags parameter defines flags indicating just how to 
//                wait. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DDVPWAIT_BEGINVBLANK -
//                    Stop waiting at the start of the VBI. 
//                  DDVPWAIT_ENDVBLANK -
//                    Stop waiting at the end of the VBI. 
//                  DDVPWAIT_LINE -
//                    Stop waiting when the line number specified in the Line 
//                    parameter of this method is passed. 
//              pLine - 
//                The Line parameter defines the video line number, relative to 
//                field height and before prescaling, that the waiting should 
//                stop. 
//                This parameter is only used when DDVPWAIT_LINE is set. 
//              pTimeout - 
//                The Timeout parameter defines the maximum time, in 
//                milliseconds, to wait before timing out and returning. 
//                Setting this property to 0 results in a wait of 3 times the 
//                setting of TDDVideoPortDesc::MicrosecondsPerField. 
// --------------------------------------------------------------------------
bool __fastcall TDx_DrawVideoPort::WaitForSync( dword pFlags, dword pLine, dword pTimeout )
{
// Original Function Definition
// HRESULT WaitForSync( 
//   DWORD dwFlags,   
//   DWORD dwLine,   
//   DWORD dwTimeout 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::WaitForSync()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTDRAWVIDEOPORT->WaitForSync( (DWORD) pFlags, (DWORD) pLine, (DWORD) pTimeout );

// Handle any Known Results
if (fErrorValue!=DD_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::WaitForSync()", TDx_Draw3D_Library_ErrorString(fErrorValue), TDx_Draw3D_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Internal Interface Access
// --------------------------------------------------------------------------
LPDIRECTDRAWVIDEOPORT __fastcall TDx_DrawVideoPort::FGetInternal_LPDIRECTDRAWVIDEOPORT()
{
return fLPDIRECTDRAWVIDEOPORT;
}
// --------------------------------------------------------------------------
LPDIRECTDRAWVIDEOPORT* __fastcall TDx_DrawVideoPort::FGetInternal_LPDIRECTDRAWVIDEOPORT_Ptr()
{
return &fLPDIRECTDRAWVIDEOPORT;
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawVideoPort::FSetInternal_LPDIRECTDRAWVIDEOPORT( LPDIRECTDRAWVIDEOPORT pLPDIRECTDRAWVIDEOPORT )
{
if (!fCreated)
    {
    fLPDIRECTDRAWVIDEOPORT = pLPDIRECTDRAWVIDEOPORT;
    fCreated = (fLPDIRECTDRAWVIDEOPORT!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_DrawVideoPort::Internal_LPDIRECTDRAWVIDEOPORT_Update()
{
fCreated = (fLPDIRECTDRAWVIDEOPORT!=NULL);
}

// --------------------------------------------------------------------------
